function [pt_i,pt_g,dp,dipref,azimref,gammaang,beta,beta_av]=drawrays_hold_on(...
xs,xr,xp,yp,dhy,zp,v0,gamma,comaz_type,v_correct,chart);
%function phy=drawrays(xs,xr,xp,yp,zp,v0,gamma,graphtype);
%  --> graphtype=1: projections and phy
%  --> graphtype=2: 3-D plot
%  --> graphtype=3: phy stats


%exemples de parametres
%wrong_vel(0,1000,700,200,500,1500,5,2,'full')
%drawrays(0,1000,700,200,500,1500,.5,2,'full')


warning off MATLAB:divideByZero
%--- compute constants ---


phis=acos((xp-xs)/sqrt((xp-xs)^2+yp^2));
azimuth_source_ray = phis/pi*180;
phir=acos((xp-xr)/sqrt((xp-xr)^2+yp^2));
azimuth_receiver_ray = phir/pi*180;
npray = 1000;

  
  %--- solve for incident angles ---
  options= optimset('Display','off','TolFun',1.e-12);
  thetas = fminbnd('mincirc',0,pi/2.0,options,xs,xp,yp,zp,v0,gamma);
  incidence_source_surface = thetas/pi*180;
  thetar = fminbnd('mincirc',0,pi/2.0,options,xr,xp,yp,zp,v0,gamma);
  incidence_receiver_surface = thetar/pi*180;
  
  %--- compute rays ---
  
  c=v0/gamma;
  
  clear ray_s_x ray_s_y ray_s_z ray_r_x ray_r_y ray_r_z

  ray_s_x = linspace(xs,xp,npray);
  ray_s_y = tan(phis) .* (ray_s_x-xs);
  ray_s_z = - c + sqrt( (c/sin(thetas))^2 - ...
      ( sqrt((ray_s_x-xs).^2+ray_s_y.^2) - c/tan(thetas) ).^2 );

  ray_r_x = linspace(xr,xp,npray);
  ray_r_y = tan(phir) .* (ray_r_x-xr);
  ray_r_z = - c + sqrt( (c/sin(thetar))^2 - ...
      ( sqrt((ray_r_x-xr).^2+ray_r_y.^2) - c/tan(thetar) ).^2 );


%========

%--- compute window limits ---

minx = min(min(ray_s_x),min(ray_r_x));
maxx = max(max(ray_s_x),max(ray_r_x));
miny = min(min(ray_s_y),min(ray_r_y));
maxy = max(max(ray_s_y),max(ray_r_y));
%minz = min(min(ray_s_z),min(ray_r_z));
minz = 0;
maxz = max(max(ray_s_z),max(ray_r_z));

%--- compute CAM phy ---

z_extra=1.1;
%nz=npray
ddz=1;
nz=zp/ddz +round(.3*zp/ddz);
dz = linspace(0,(nz-1)*ddz,nz);
v = v0 + gamma.*dz;
%clf
%v = 0.97 .* v;
%v0= .97 * v0;

v_ca=1.*v;

for iz=1:nz-1
  v_ca(iz)=1.*(.5*(v(iz+1)+v(iz)));
end
v_ca(nz)=v(nz);

psx = sin(thetas)*cos(phis)/v0;
prx = sin(thetar)*cos(phir)/v0;

pmx = prx+psx;
phx = prx-psx;

psy = sin(thetas)*sin(phis)/v0;
pry = sin(thetar)*sin(phir)/v0;

pmy = pry+psy;
phy = pry-psy;

phx_full=phx;
phy_full=phy;
freq=30;
khy_full_30hz=2*pi*phy_full*freq;


clear ray_s_t ray_r_t;
ray_s_t(1)=0.;
ray_r_t(1)=0.;

clear v_ray_r v_ray_s v_s_ray v_r_ray
for iray =1:npray-1

   ddz_s_ray=ray_s_z(iray+1)-ray_s_z(iray);
   ddz_r_ray=ray_r_z(iray+1)-ray_r_z(iray);

if(v_correct==2)
   v_r_ray= 1410+ .470*.5*(ray_r_z(iray+1)+ray_r_z(iray));
   v_s_ray= 1410+ .470*.5*(ray_s_z(iray+1)+ray_s_z(iray));
end
if(v_correct==1)
   v_r_ray= v0+ gamma*.5*(ray_r_z(iray+1)+ray_r_z(iray));
   v_s_ray= v0+ gamma*.5*(ray_s_z(iray+1)+ray_s_z(iray));
end
if(v_correct==0)
   v_r_ray= 1425+ .475*.5*(ray_r_z(iray+1)+ray_r_z(iray));
   v_s_ray= 1425+ .475*.5*(ray_s_z(iray+1)+ray_s_z(iray));
end

   v_ray_s(iray) = v_s_ray;
   v_ray_r(iray) = v_r_ray;

   psz = sqrt((1/v_s_ray)^2 - psx^2 - psy^2);
   prz = sqrt((1/v_r_ray)^2 - prx^2 - pry^2);

   dst = ddz_s_ray/(psz*v_s_ray^2);
   drt = ddz_r_ray/(prz*v_r_ray^2);

   ray_s_t(iray+1)=ray_s_t(iray) + dst;
   ray_r_t(iray+1)=ray_r_t(iray) + drt;
end
if(v_correct==2)
   v_r_ray= 1410+ .47*.5*(ray_r_z(iray+1)+ray_r_z(iray));
   v_s_ray= 1410+ .47*.5*(ray_s_z(iray+1)+ray_s_z(iray));
end
if(v_correct==1)
v_ray_r(npray)= v0+ gamma*(ray_r_z(npray));
v_ray_s(npray)= v0+ gamma*(ray_s_z(npray));
end
if(v_correct==0)
v_ray_r(npray)= 1425+ .475*(ray_r_z(npray));
v_ray_s(npray)= 1425+ .475*(ray_s_z(npray));
end

s_t=ray_s_t(npray);
r_t=ray_r_t(npray);
tot_ray_t=s_t+r_t;

% range of phy for narrow azimuth
v_min=1800;
v_max=2150;

v_min=min([v_min abs(1./psx) abs(1./prx)]);

rs = sqrt( 1-(v_min^2)*psx^2 );
rr = sqrt( 1-(v_min^2)*prx^2 );

if((rr + rs) > 0.)
  phy_min = pmy * (rr-rs)/(rr+rs);
else
  phy_min = 0.;
end

v_max_old=v_max;
v_max=min([v_max_old abs(1./psx) abs(1./prx)]);
if(v_max < v_max_old) 
  display(v_max)
  display(v_max_old)
  warning('HAD TO LOWER VMAX')
end

rs = sqrt( 1-(v_max^2)*psx^2 );
rr = sqrt( 1-(v_max^2)*prx^2 );

if((rr + rs) > 0.)
  phy_max = pmy * (rr-rs)/(rr+rs);
else
  phy_max = 0;
end


clear ray_s_x_ca ray_s_y_ca ray_r_z_ca ray_r_x_ca ray_r_y_ca
clear ray_s_t_ca ray_r_t_ca 
clear psy_ca pry_ca psz_ca prz_ca ray_s_z_ca 

ray_s_z_ca(1)=0.;
ray_s_x_ca(1)=ray_s_x(1);
ray_s_y_ca(1)=ray_s_y(1);
ray_s_t_ca(1)=0.;

ray_r_z_ca(1)=0.;
ray_r_x_ca(1)=ray_r_x(1);
ray_r_y_ca(1)=ray_r_y(1);
ray_r_t_ca(1)=0.;

tot_ray_t_ca=0.;
tot_ray_t_ca_test=0.;
iz=0;

%for iz =1:nz
cnt=1;
while ( (iz < (nz-1) ) & (tot_ray_t_ca_test < tot_ray_t))
%for iz =1:3

   iz=iz+1;

   if((1-(v_ca(iz)^2)*psx^2) < 0 )
     error('SZ RAY TURNED')
   end
   roots(iz) = sqrt( 1-(v_ca(iz)^2)*psx^2 );
   if((1-(v_ca(iz)^2)*prx^2) < 0 )
     error('RX RAY TURNED')
   end
   rootr(iz) = sqrt( 1-(v_ca(iz)^2)*prx^2 );


   switch lower(comaz_type) 
     case ('full')
%       warning('Exact downward continuation'),
       phy_ca(iz) = phy;
     case ('opw')
%       warning('OPW downward continuation'),
       phy_ca(iz) = 0.;
     case ('comaz')
%       warning('Common Azimuth downward continuation'),
       phy_ca(iz) = pmy * (rootr(iz)-roots(iz))/(rootr(iz)+roots(iz));
     otherwise
       comaz_type
       error('comaz_type needs to be passed')
    end

   psy_ca(iz)=.5*(pmy-phy_ca(iz));
   pry_ca(iz)=.5*(pmy+phy_ca(iz));

   arg=1/(v_ca(iz)^2) - psx^2 - psy_ca(iz)^2;
   if(arg < 0 )
     hx
     tot_ray_t;
     tot_ray_t_ca;
     error_z=dz(iz)
     error('S RAY TURNED')
   end
   psz_ca(iz) =  sqrt(arg);

   arg=1/(v_ca(iz)^2) - prx^2 - pry_ca(iz)^2;
   if(arg < 0 )
     error_z=dz(iz)
     error('R RAY TURNED')
   end
   prz_ca(iz) =  sqrt(arg);

   dsx=ddz*psx/psz_ca(iz);
   dsy=ddz*psy_ca(iz)/psz_ca(iz);

   drx=ddz*prx/prz_ca(iz);
   dry=ddz*pry_ca(iz)/prz_ca(iz);

   dst=ddz/(psz_ca(iz)*v_ca(iz)^2);
   drt=ddz/(prz_ca(iz)*v_ca(iz)^2);


   if(iz < nz) 
     ray_s_z_ca(iz+1)=ray_s_z_ca(iz) + ddz;
     ray_s_x_ca(iz+1)=ray_s_x_ca(iz) + dsx;
     ray_s_y_ca(iz+1)=ray_s_y_ca(iz) + dsy;
     ray_s_t_ca(iz+1)=ray_s_t_ca(iz) + dst;
 
     ray_r_z_ca(iz+1)=ray_r_z_ca(iz) + ddz;
     ray_r_x_ca(iz+1)=ray_r_x_ca(iz) + drx;
     ray_r_y_ca(iz+1)=ray_r_y_ca(iz) + dry;
     ray_r_t_ca(iz+1)=ray_r_t_ca(iz) + drt;

     hx=.5*(ray_r_x_ca(iz)-ray_s_x_ca(iz));

     tot_ray_t_ca=ray_s_t_ca(iz+1)+ray_r_t_ca(iz+1) - phx*hx;

     tot_ray_t_ca_test=tot_ray_t_ca; 
     
     if(mod(iz,100)==00)
	sx_dis(cnt)=ray_s_x_ca(iz);
	sy_dis(cnt)=ray_s_y_ca(iz);
	rx_dis(cnt)=ray_r_x_ca(iz);
	ry_dis(cnt)=ray_r_y_ca(iz);
	cnt=cnt+1;
     end
     %AZIMUTH
     refl_azim(iz)=atan((ray_r_y_ca(iz)-ray_s_y_ca(iz))/...
                        (ray_r_x_ca(iz)-ray_s_x_ca(iz)))*180/pi;
   end

end
     
     %---source and receiver vector
     [junk,nn]=size(ray_s_x_ca);
     vec_s=[ray_s_x_ca(nn)-ray_s_x_ca(nn-1) ...
            ray_s_y_ca(nn)-ray_s_y_ca(nn-1) ...
            ray_s_z_ca(nn)-ray_s_z_ca(nn-1)];
     vec_s=vec_s/norm(vec_s);
     vec_r=[ray_r_x_ca(nn)-ray_r_x_ca(nn-1) ...
            ray_r_y_ca(nn)-ray_r_y_ca(nn-1) ...
            ray_r_z_ca(nn)-ray_r_z_ca(nn-1)];
     vec_r=vec_r/norm(vec_r);
%VECTOR OF THE NORMAL
     vec_n=vec_s+vec_r;
     vec_n=vec_n/norm(vec_n);
%ANGLES OF THE REFLECTOR
     [dipref,azimref]=vec2angle(vec_n);
      dipref=90-dipref; % dip_plane = 90 - dip_normal;
      azimref=90-azimref; % idem;
%REFLECTION AZIMUTH
     [junk,nn]=size(refl_azim);
     beta=refl_azim(nn);
     beta_av=refl_azim(nn-9:nn);
     beta_av=sum(beta_av)/10;
%APERTURE ANGLE
     gammaang=acos(dot(vec_s,vec_r))*180/pi/2;
%if(1)
%COORDINATES
     %disp('#####COORDINATES OF S POINT#####')
     pt_s=[ray_s_x_ca(nn) ray_s_y_ca(nn) ray_s_z_ca(nn)];
     %disp('#####COORDINATES OF R POINT#####')
     pt_r=[ray_r_x_ca(nn) ray_r_y_ca(nn) ray_r_z_ca(nn)];

%IMAGE POINT IN THE OFFSET DOMAIN
     pt_i=(pt_s+pt_r)/2;

%ANGLES OF THE RAYS
     %disp('#####ANGLES OF R RAY#####')
     [dip_s,azim_s]=vec2angle(vec_s);
     [dip_r,azim_r]=vec2angle(vec_r);

%COORDINATES OF IMAGE POINT IN ANGLE DOMAIN
     %disp('#####COORD OF I_gamma #####')
     pt_g=image_position(pt_s(1),pt_s(2),pt_s(3),pt_r(1),pt_r(2),...
     dip_s,azim_s,dip_r,azim_r);
%SHIFT
     dp=pt_g-pt_i
%end

tot_ray_t_ca;
tot_ray_t;

eps_z=(tot_ray_t_ca-tot_ray_t)/(drt+dst);

tot_ray_t_ca=tot_ray_t_ca-eps_z*(dst+drt);

iz_final=iz+1;
z_s_final=ray_s_z_ca(iz_final)-eps_z*ddz;
z_r_final=ray_r_z_ca(iz_final)-eps_z*ddz;
x_s_final=ray_s_x_ca(iz_final)-eps_z*dsx;
x_r_final=ray_r_x_ca(iz_final)-eps_z*drx;
y_s_final=ray_s_y_ca(iz_final)-eps_z*dsy;
y_r_final=ray_r_y_ca(iz_final)-eps_z*dry;

ray_s_z_ca(iz_final)=z_s_final;
ray_r_z_ca(iz_final)=z_r_final;
ray_s_x_ca(iz_final)=x_s_final;
ray_r_x_ca(iz_final)=x_r_final;
ray_s_y_ca(iz_final)=y_s_final;
ray_r_y_ca(iz_final)=y_r_final;

hx_final=.5*(x_r_final-x_s_final);

x_ref_ca=x_s_final+hx_final;
y_ref_ca=.5*(y_s_final+y_r_final);
z_ref_ca=.5*(z_s_final+z_r_final);

delta_z=z_ref_ca-zp;
delta_x=x_ref_ca-xp;
delta_y=y_ref_ca-yp;


%--- compute reflector plane ---

turn_s = abs(ray_s_z(npray)-max(ray_s_z)) > 0.0001;
if(turn_s) 
warning('FULL S RAY TURNED')
  b=ray_s_z(npray)
  [m, mp] =max(ray_s_z)
  clear psz_p_Z 
end
turn_r = abs(ray_r_z(npray)-max(ray_r_z)) > 0.0001;
if(turn_r) 
warning('FULL R RAY TURNED')
  b=ray_r_z(npray);
  m=max(ray_r_z);
end

npray
size(v_ray_s)
v_ray_ref=v_ray_s(npray-1);

psz_p = (1-2*turn_s) * sqrt( 1/v_ray_ref^2 - psx^2 - psy^2 );
prz_p = (1-2*turn_r) * sqrt( 1/v_ray_ref^2 - prx^2 - pry^2 );


norm_x = prx+psx;
norm_y = pry+psy;
norm_z = prz_p+psz_p;
norm_ = sqrt(norm_x^2+norm_y^2+norm_z^2);

norm_x = norm_x/norm_;
norm_y = norm_y/norm_;
norm_z = norm_z/norm_;

ref_x=norm_x;
ref_y=norm_y;
ref_z=norm_z;

dip_refl = acos(norm_z);
azim_refl = acos(norm_x/sin(dip_refl));

azimuth_reflector = azim_refl/pi*180;
dip_reflector = dip_refl/pi*180;

trac_ps_x = linspace(ray_s_x(npray),ray_s_x(npray)+1e6*psx,50);
trac_ps_y = linspace(ray_s_y(npray),ray_s_y(npray)+1e6*psy,50);
trac_ps_z = linspace(ray_s_z(npray),ray_s_z(npray)+1e6*psz_p,50);

trac_pr_x = linspace(ray_s_x(npray),ray_r_x(npray)+1e6*prx,50);
trac_pr_y = linspace(ray_s_y(npray),ray_r_y(npray)+1e6*pry,50);
trac_pr_z = linspace(ray_s_z(npray),ray_r_z(npray)+1e6*prz_p,50);


trac_norm_x = linspace(ray_s_x(npray),ray_s_x(npray)+5e5*norm_*norm_x,50);
trac_norm_y = linspace(ray_s_y(npray),ray_s_y(npray)+5e5*norm_*norm_y,50);
trac_norm_z = linspace(ray_s_z(npray),ray_s_z(npray)+5e5*norm_*norm_z,50);

trac_ref_x = linspace(ray_s_x(npray),ray_s_x(npray)+5e5*norm_*ref_x,50);
trac_ref_y = linspace(ray_s_y(npray),ray_s_y(npray)+5e5*norm_*ref_y,50);
trac_ref_z = linspace(ray_s_z(npray),ray_s_z(npray)+5e5*norm_*ref_z,50);


z_ref_ca_plane=zp-((x_ref_ca-xp)*ref_x + (y_ref_ca-yp)*ref_y)/ref_z;

delta_z_plane=z_ref_ca-z_ref_ca_plane;

% find max cross-line offset

if (lower(comaz_type(1:3)) ~= 'opw') 
iz=0;
not_found=1;
while (( iz < nz) & not_found)
  iz=iz+1;
  if(phy_ca(iz) >= phy)
    not_found=0;
  end
end
z_max=(iz-1)*ddz;

iray=0;
not_found=1;
while ((iray < npray) & not_found)
  iray=iray+1;
  if(ray_s_z(iray) > z_max)
    not_found=0;
  end
end
ys_max=ray_s_y(iray);

iray=0;
not_found=1;
while ((iray < npray) & not_found)
  iray=iray+1;
  if(ray_r_z(iray) > z_max)
    not_found=0;
  end
end
yr_max=ray_r_y(iray);

hy_max=.5*(yr_max-ys_max);
end


phx_full=phx;

if(chart==0)
%--- begin plots ---

  hold on
  grid on
  axis([(minx-1600) (maxx+500) -(maxy-900) (maxy+50) -(maxz+500) -minz])
  campos(10000.*[0.4881    0.7015    1.3026])

  plot3(xs,0-dhy,0.,'b*','LineWidth',[2.]);
  plot3(xr,0-dhy,0.,'ro','LineWidth',[2.]);
  plot3(xp,yp-dhy,-zp,'dy','LineWidth',[2.]);
  plot3(pt_g(1),pt_g(2)-dhy,-pt_g(3),'kp','LineWidth',[2.])

% r ray
  plot3(ray_r_x_ca,ray_r_y_ca-dhy,-ray_r_z_ca,'r-','LineWidth',[2.5]) % const exmp1

% s ray
  plot3(ray_s_x_ca,ray_s_y_ca-dhy,-ray_s_z_ca,'b-','LineWidth',[1.])
  set(gca,'Box','on','LineWidth',[2.],'GridLineStyle',':')
  set(gca,'ZTick',[-1600 -1400 -1200 -1000 -800 -600 -400 -200 0])

  legend('Sources','Receivers','Reflector','Image point in the angle-domain','Receiver ray','Source ray',4)

xlabel('Inline CMP (m)','FontSize',[14]);
ylabel('Crossline CMP (m)','FontSize',[14]);
zlabel('Depth (m)','FontSize',[14]);
end
