function hvocig2D(xs,zs,xr,gamma,alpha)

%(xs,zs) coordinates of the end point of source ray
% xr horizontal coordinate of the end point of receiver ray 
% gamma: aperture angle in degrees
% alpha: interface dipping angle in degrees

%examples of values hvocig2D(1000,1000,1500,20,35)

nbpts = 20 ;    %number of points to draw lines

%conversion in radian of alpha and gamma angles
alpha = alpha * pi / 180 ;
gamma = gamma * pi / 180 ;

%<<<<==== COMPUTING SOURCE AND RECEIVER RAY ====>>>>

%---- ray vectors for source, receiver, normal and interface
vecs = [ -sin(alpha+gamma) cos(alpha+gamma) ] ;
vecr = [ -sin(alpha-gamma) cos(alpha-gamma) ] ;
vecn = vecs + vecr ; %normal ray direction
vecn = vecn / norm(vecn) ; %normalization
veci = [ vecn(2) -vecn(1) ] ; %interface vector direction

%---- coordinates of some points
zr = zs + ( xr - xs ) * tan(alpha) ; %receiver end point Ro
xo = ( xs + xr ) / 2 ; % middle of SoRo: Io
zo = ( zs + zr ) / 2 ; % middle of SoRo: Io

%---- computing the rays and the interface
dots = linspace(-zs/3*2,zs/2,nbpts) ;
%--- source ray
line_source_x    = xs + vecs(1) * dots ;
line_source_z    = zs + vecs(2) * dots ;
%--- receiver ray
line_receiver_x  = xr + vecr(1) * dots ;
line_receiver_z  = zr + vecr(2) * dots ;
%--- normal ray
line_normal_x    = xo + vecn(1) * dots ;
line_normal_z    = zo + vecn(2) * dots ;
%--- interface
line_interface_x = xs + linspace(0,xr-xs,nbpts) ;
line_interface_z = zs + linspace(0,zr-zs,nbpts) ;

%<<<<==== COMPUTING HOCIG ====>>>>

%Coordinates of Sxh, Rxh and Ixh
zh  =  (zr*cos(alpha+gamma)+zs*cos(alpha-gamma))/...
       (   cos(alpha+gamma)+   cos(alpha-gamma))    ;
xsh = xs + tan(alpha+gamma) * (zs-zh) ;
xrh = xr + tan(alpha-gamma) * (zr-zh) ;
xih = ( xsh + xrh ) / 2 ; %Ixh middle of [Sxh Rxh]

%--- [ Sxh Rxh ] segment
line_ho_x = linspace(xsh,xrh,nbpts) ;
line_ho_z = zh + zeros(nbpts) ;

%<<<<==== COMPUTING VOCIG ====>>>>

xv  =  (xr*sin(alpha+gamma)+xs*sin(alpha-gamma))/...
       (   sin(alpha+gamma)+   sin(alpha-gamma))    ;
zsv = zs + 1/tan(alpha+gamma) * (xs-xv) ;
zrv = zr + 1/tan(alpha-gamma) * (xr-xv) ;
ziv = ( zsv + zrv ) / 2 ; %Izv middle of [Szv Rzv]

%--- [ Szv Rzv ] segment
line_vo_z = linspace(zsv,zrv,nbpts) ;
line_vo_x = xv + zeros(nbpts) ;

%<<<<==== COMPUTING ADCIG ====>>>>

%coordinates of I_gamma (ig)

%compute the line orthogonal to the HOCIG passing through Ixh
line_igh_x = zeros(8) + xih ;
line_igh_z = linspace(zh,ziv,8);

%compute the line orthogonal to the VOCIG passing through Izh
line_igv_x = linspace(xih,xv,8);
line_igv_z = zeros(8) + ziv ;



%<<<<==== PLOTTING EVERYTHING ====>>>>
clf
hold on
%--- given geometry of the problem
plot(xs,zs,'r*') %So
plot(xr,zr,'bv') %Ro
plot(xo,zo,'ko') %Io
plot(line_interface_x,line_interface_z,'k') %interface
plot(line_normal_x   ,line_normal_z   ,'k') %normal
plot(line_source_x   ,line_source_z   ,'r') %source ray
plot(line_receiver_x ,line_receiver_z ,'b') %receiver ray

%--- HOCIG
plot(xsh,zh,'r*')    %Sxh
plot(xrh,zh,'bv')    %Rxh
plot(xih,zh,'ko')    %Ixh
plot(line_ho_x,line_ho_z,'g') %[ Sxh Rxh ] segment

%--- HOCIG
%plot(xv,zsv,'r*')    %Szv
%plot(xv,zrv,'bv')    %Rzv
%plot(xv,ziv,'ko')    %Izv
%plot(line_vo_x,line_vo_z,'m') %[ Szv Rzv ] segment

%--- ADCIG
%plot(line_igh_x,line_igh_z,'k.') %[ Ihx Ig ] segment
%plot(line_igv_x,line_igv_z,'k.') %[ Ihv Ig ] segment

axis equal
grid
xlabel('x')
ylabel('z')
%title('2D HOCIG and VOCIG with wrong velocity ')
