function hvocig3dbis(xs,ys,zs,xr,yr,thetas,azims,thetar,azimr)

%(xs,ys,zs) coordinates of the end point of source   ray
%(xr,yr,zr) coordinates of the end point of receiver ray
%thetas,azims: dippnig and azimuth of source ray at endind point
%thetar,azimr: dippnig and azimuth of receiver ray at endind point

%ex of values (1000,1000,1000,1500,900,30,30,60,60)

nbpts = 20 ;    %number of points to draw lines

%conversion in radian of alpha and gamma angles
thetas = thetas * pi / 180 ;
thetar = thetar * pi / 180 ;
azimr  = azimr  * pi / 180 ;
azims  = azims  * pi / 180 ;

%<<<<==== COMPUTING SOURCE AND RECEIVER RAY ====>>>>

%---- ray vectors for source, receiver, normal
vecs = [ -cos(thetas)*cos(azims)...
          cos(thetas)*sin(azims)...
          sin(thetas) ] ;
vecr = [ -cos(thetar)*cos(azimr)...
          cos(thetar)*sin(azimr)...
          sin(thetar) ] ;

vecn = vecs + vecr ; %normal ray direction
vecn = vecn / norm(vecn) ; %normalization

%--- Window
xmax=max(xr,xs)+100;
ymax=max(yr,ys)+100;
xmin=min(xr,xs)-100;
ymin=min(yr,ys)-100;

%--- Computation of the plane orthogonal to normal and containing S
x=linspace(xmin,xmax,5);
y=linspace(ymin,ymax,5);
[plane_x,plane_y]=meshgrid(x,y);
 plane_z = zs - (vecn(1)*(plane_x-xs)+...
                 vecn(2)*(plane_y-ys) )/vecn(3);

%---- coordinates of some points
zr = zs - ( ( xr - xs ) * vecn(1) + ...
            ( yr - ys ) * vecn(2)      ) / vecn(3); %receiver end point Ro
xo = ( xs + xr ) / 2 ; % middle of SoRo: Io
yo = ( ys + yr ) / 2 ; % middle of SoRo: Io
zo = ( zs + zr ) / 2 ; % middle of SoRo: Io

%---- computing the rays and the interface
dots = linspace(-600,600,nbpts) ;
%--- source ray
line_source_x    = xs + vecs(1) * dots ;
line_source_y    = ys + vecs(2) * dots ;
line_source_z    = zs + vecs(3) * dots ;
%--- receiver ray
line_receiver_x  = xr + vecr(1) * dots ;
line_receiver_y  = yr + vecr(2) * dots ;
line_receiver_z  = zr + vecr(3) * dots ;
%--- normal ray
line_normal_x    = xo + vecn(1) * dots ;
line_normal_y    = yo + vecn(2) * dots ;
line_normal_z    = zo + vecn(3) * dots ;
%--- interface
line_interface_x = xs + linspace(0,xr-xs,nbpts) ;
line_interface_y = ys + linspace(0,yr-ys,nbpts) ;
line_interface_z = zs + linspace(0,zr-zs,nbpts) ;

%<<<<==== COMPUTING HOCIG ====>>>>

%Coordinates of Sxh, Rxh and Ixh
zh  =  ( zr*sin(thetas) + zs*sin(thetar) ) /...
       (    sin(thetas) +    sin(thetar) )    ;

xsh = xs + 1/tan(thetas) * (zs-zh) * cos(azims);
xrh = xr + 1/tan(thetar) * (zr-zh) * cos(azimr);
xih = ( xsh + xrh ) / 2 ; %Ixh middle of [Sxh Rxh]

ysh = ys - 1/tan(thetas) * (zs-zh) * sin(azims);
yrh = yr - 1/tan(thetar) * (zr-zh) * sin(azimr);
yih = ( ysh + yrh ) / 2 ; %Ixh middle of [Sxh Rxh]

%--- [ Sxh Rxh ] segment
line_ho_x = linspace(xsh,xrh,nbpts) ;
line_ho_y = linspace(ysh,yrh,nbpts) ;
line_ho_z = zh + zeros(nbpts) ;

%<<<<==== COMPUTING VOCIG for x ====>>>>

xv  =  (xr*cos(azims)*cos(thetas)+xs*cos(azimr)*cos(thetar))/...
       (   cos(azims)*cos(thetas)+   cos(azimr)*cos(thetar))    ;

zsv = zs + tan(thetas) * (xs-xv) / cos(azims);
zrv = zr + tan(thetar) * (xr-xv) / cos(azimr);
ziv = ( zsv + zrv ) / 2 ; %Izv middle of [Szv Rzv]

ysv = ys + tan(azims) * (xs-xv);
yrv = yr + tan(azimr) * (xr-xv);
yiv = ( ysv + yrv ) / 2 ; %Ixh middle of [Szv Rzv]

%--- [ Szv Rzv ] segment
line_vo_x = xv + zeros(nbpts/4) ;
line_vo_y = linspace(ysv,yrv,nbpts/4) ;
line_vo_z = linspace(zsv,zrv,nbpts/4) ;

%<<<<==== COMPUTING VOCIG for y ====>>>>

yv  =  (yr*sin(azims)*cos(thetas)+ys*sin(azimr)*cos(thetar))/...
       (   sin(azims)*cos(thetas)+   sin(azimr)*cos(thetar))    ;

zsv2 = zs - tan(thetas) * (ys-yv) / sin(azims);
zrv2 = zr - tan(thetar) * (yr-yv) / sin(azimr);
ziv2 = ( zsv2 + zrv2 ) / 2;  %Izv2 middle of [Szv2 Rzv2]

xsv = xs + 1/tan(azims) * (ys-yv);
xrv = xr + 1/tan(azimr) * (yr-yv);
xiv2 = ( xsv  + xrv  ) / 2;  %Ixh2 middle of [Szv2 Rzv2]

%--- [ Szv2 Rzv2 ] segment
line_vo_y2 = yv + zeros(nbpts/4) ;
line_vo_x2 = linspace(xsv ,xrv ,nbpts/4) ;
line_vo_z2 = linspace(zsv2,zrv2,nbpts/4) ;

figure(1)

%<<<<==== PLOTTING EVERYTHING ====>>>>		3D
%--- given geometry of the problem
subplot(1,2,1),plot3(xs,ys,zs,'r*') %So
hold on
plot3(xr,yr,zr,'bv') %Ro
plot3(xo,yo,zo,'ko') %Io
plot3(line_interface_x,line_interface_y,line_interface_z,'k') %interface
%mesh(plane_x,plane_y,plane_z)
%plot3(line_normal_x   ,line_normal_y   ,line_normal_z   ,'k') %normal
plot3(line_source_x   ,line_source_y   ,line_source_z   ,'r') %source ray
plot3(line_receiver_x ,line_receiver_y ,line_receiver_z ,'b') %source ray

%--- HOCIG
plot3(xsh,ysh,zh,'r*') %Sxh
plot3(xrh,yrh,zh,'bv') %Rxh
plot3(xih,yih,zh,'ko') %Ixh
plot3(line_ho_x,line_ho_y,line_ho_z,'k.') %[Sxh Rxh] segment

%--- VOCIG - x 
plot3(xv,ysv,zsv,'r*') %Szv
plot3(xv,yrv,zrv,'bv') %Rzv
plot3(xv,yiv,ziv,'ko') %Izv
plot3(line_vo_x,line_vo_y,line_vo_z,'k--') %[Szv Rzv] segment

%--- VOCIG - y
plot3(xsv,yv,zsv2,'r*') %Szv2
plot3(xrv,yv,zrv2,'bv') %Rzv2
plot3(xiv2,yv,ziv2,'ko') %Izv2
plot3(line_vo_x2,line_vo_y2,line_vo_z2,'k-.') %[Szv2 Rzv2] segment

%--- Display
xmax=max(max(xv,max(xrh,xsh)),max(xr,xs))+200;
xmin=min(min(xv,min(xrh,xsh)),min(xr,xs))-150;
zmax=max(max(zh,max(zrv,zsv)),max(zr,zs))+200;
zmin=min(min(zh,min(zrv,zsv)),min(zr,zs))-150;
axis([xmin xmax ymin ymax zmin zmax])
axis equal
grid
xlabel('x')
ylabel('y')
zlabel('z')
title('3D HOCIG and VOCIG with wrong velocity ')

%<<<<==== PLOTTING EVERYTHING ====>>>>		x
%--- given geometry of the problem
subplot(1,2,2),plot3(xs,ys,zs,'r*') %So
hold on
plot3(xr,yr,zr,'bv') %Ro
plot3(xo,yo,zo,'ko') %Io
plot3(line_interface_x,line_interface_y,line_interface_z,'k') %interface
%mesh(plane_x,plane_y,plane_z)
%plot3(line_normal_x   ,line_normal_y   ,line_normal_z   ,'k') %normal
plot3(line_source_x   ,line_source_y   ,line_source_z   ,'r') %source ray
plot3(line_receiver_x ,line_receiver_y ,line_receiver_z ,'b') %source ray

%--- HOCIG
plot3(xsh,ysh,zh,'r*') %Sxh
plot3(xrh,yrh,zh,'bv') %Rxh
plot3(xih,yih,zh,'ko') %Ixh
plot3(line_ho_x,line_ho_y,line_ho_z,'k.') %[Sxh Rxh] segment

%--- VOCIG - x 
plot3(xv,ysv,zsv,'r*') %Szv
plot3(xv,yrv,zrv,'bv') %Rzv
plot3(xv,yiv,ziv,'ko') %Izv
plot3(line_vo_x,line_vo_y,line_vo_z,'k--') %[Szv Rzv] segment

%--- VOCIG - y
plot3(xsv,yv,zsv2,'r*') %Szv2
plot3(xrv,yv,zrv2,'bv') %Rzv2
plot3(xiv2,yv,ziv2,'ko') %Izv2
plot3(line_vo_x2,line_vo_y2,line_vo_z2,'k-.') %[Szv2 Rzv2] segment

%--- Display
xmax=max(max(xv,max(xrh,xsh)),max(xr,xs))+200;
xmin=min(min(xv,min(xrh,xsh)),min(xr,xs))-150;
zmax=max(max(zh,max(zrv,zsv)),max(zr,zs))+200;
zmin=min(min(zh,min(zrv,zsv)),min(zr,zs))-150;
axis([xmin xmax ymin ymax zmin zmax])
axis equal
grid
xlabel('x')
ylabel('y')
zlabel('z')
title('3D HOCIG and VOCIG with wrong velocity ')

figure(2)

%<<<<==== PLOTTING EVERYTHING ====>>>>		y
%--- given geometry of the problem
subplot(1,2,1),plot3(xs,ys,zs,'r*') %So
hold on
plot3(xr,yr,zr,'bv') %Ro
plot3(xo,yo,zo,'ko') %Io
plot3(line_interface_x,line_interface_y,line_interface_z,'k') %interface
%mesh(plane_x,plane_y,plane_z)
%plot3(line_normal_x   ,line_normal_y   ,line_normal_z   ,'k') %normal
plot3(line_source_x   ,line_source_y   ,line_source_z   ,'r') %source ray
plot3(line_receiver_x ,line_receiver_y ,line_receiver_z ,'b') %source ray

%--- HOCIG
plot3(xsh,ysh,zh,'r*') %Sxh
plot3(xrh,yrh,zh,'bv') %Rxh
plot3(xih,yih,zh,'ko') %Ixh
plot3(line_ho_x,line_ho_y,line_ho_z,'k.') %[Sxh Rxh] segment

%--- VOCIG - x 
plot3(xv,ysv,zsv,'r*') %Szv
plot3(xv,yrv,zrv,'bv') %Rzv
plot3(xv,yiv,ziv,'ko') %Izv
plot3(line_vo_x,line_vo_y,line_vo_z,'k--') %[Szv Rzv] segment

%--- VOCIG - y
plot3(xsv,yv,zsv2,'r*') %Szv2
plot3(xrv,yv,zrv2,'bv') %Rzv2
plot3(xiv2,yv,ziv2,'ko') %Izv2
plot3(line_vo_x2,line_vo_y2,line_vo_z2,'k-.') %[Szv2 Rzv2] segment

%--- Display
xmax=max(max(xv,max(xrh,xsh)),max(xr,xs))+200;
xmin=min(min(xv,min(xrh,xsh)),min(xr,xs))-150;
zmax=max(max(zh,max(zrv,zsv)),max(zr,zs))+200;
zmin=min(min(zh,min(zrv,zsv)),min(zr,zs))-150;
axis([xmin xmax ymin ymax zmin zmax])
axis equal
grid
xlabel('x')
ylabel('y')
zlabel('z')
title('3D HOCIG and VOCIG with wrong velocity ')

%<<<<==== PLOTTING EVERYTHING ====>>>>		z
%--- given geometry of the problem
subplot(1,2,2),plot3(xs,ys,zs,'r*') %So
hold on
plot3(xr,yr,zr,'bv') %Ro
plot3(xo,yo,zo,'ko') %Io
plot3(line_interface_x,line_interface_y,line_interface_z,'k') %interface
%mesh(plane_x,plane_y,plane_z)
%plot3(line_normal_x   ,line_normal_y   ,line_normal_z   ,'k') %normal
plot3(line_source_x   ,line_source_y   ,line_source_z   ,'r') %source ray
plot3(line_receiver_x ,line_receiver_y ,line_receiver_z ,'b') %source ray

%--- HOCIG
plot3(xsh,ysh,zh,'r*') %Sxh
plot3(xrh,yrh,zh,'bv') %Rxh
plot3(xih,yih,zh,'ko') %Ixh
plot3(line_ho_x,line_ho_y,line_ho_z,'k.') %[Sxh Rxh] segment

%--- VOCIG - x 
plot3(xv,ysv,zsv,'r*') %Szv
plot3(xv,yrv,zrv,'bv') %Rzv
plot3(xv,yiv,ziv,'ko') %Izv
plot3(line_vo_x,line_vo_y,line_vo_z,'k--') %[Szv Rzv] segment

%--- VOCIG - y
plot3(xsv,yv,zsv2,'r*') %Szv2
plot3(xrv,yv,zrv2,'bv') %Rzv2
plot3(xiv2,yv,ziv2,'ko') %Izv2
plot3(line_vo_x2,line_vo_y2,line_vo_z2,'k-.') %[Szv2 Rzv2] segment

%--- Display
xmax=max(max(xv,max(xrh,xsh)),max(xr,xs))+200;
xmin=min(min(xv,min(xrh,xsh)),min(xr,xs))-150;
zmax=max(max(zh,max(zrv,zsv)),max(zr,zs))+200;
zmin=min(min(zh,min(zrv,zsv)),min(zr,zs))-150;
axis([xmin xmax ymin ymax zmin zmax])
axis equal
grid
xlabel('x')
ylabel('y')
zlabel('z')
title('3D HOCIG and VOCIG with wrong velocity ')

