function hvocig3Dbut(xs,ys,zs,xr,yr,...
                     thetas,azims,thetar,azimr)

%(pt_s(1),pt_s(2),pt_s(3)) coordinates of the end point of source   ray
%(pt_r(1),pt_r(2),pt_r(3)) coordinates of the end point of receiver ray
%thetas,azims: dip and azimuth of source ray at end point
%thetar,azimr: dip and azimuth of receiver ray at end point

%examples of values (1000,1000,1000,1500,900,20,30,60,80)
%examples of values hvocig3Dbut(1000,1200,1000,1250,900,25,20,55,70)

clf
hold on

pt_s=[xs ys zs];
pt_r(1)=xr;
pt_r(2)=yr;

nbpts = 20 ;    %number of points to draw lines

%conversion in radian of alpha and gamma angles
thetas = thetas * pi / 180 ;
thetar = thetar * pi / 180 ;
azimr  = azimr  * pi / 180 ; 
azims  = azims  * pi / 180 ;

%|||||||||||||||||||||||||||||||||||||
%|||||||||||||||||||||||||||||||||||||
%<<<<===== ORIGINAL GEOMETRY =====>>>>
%|||||||||||||||||||||||||||||||||||||
%|||||||||||||||||||||||||||||||||||||

%<<<<==== SOURCE AND RECEIVER RAY ====>>>>

%---- ray vectors for source, receiver, normal
vecs=angle2vec(thetas,azims);
vecr=angle2vec(thetar,azimr);

vecn = vecs + vecr ; %normal ray direction
vecn = vecn / norm(vecn) ; %normalization

%--- Window
xmax=max(pt_r(1),pt_s(1))+100;
ymax=max(pt_r(2),pt_s(2))+100;
xmin=min(pt_r(1),pt_s(1))-100;
ymin=min(pt_r(2),pt_s(2))-100;

%--- the plane orthogonal to normal and containing S
x=linspace(xmin,xmax,10);
y=linspace(ymin,ymax,10);
[plane_x,plane_y]=meshgrid(x,y);
 plane_z = pt_s(3) - (vecn(1)*(plane_x-pt_s(1))+...
                 vecn(2)*(plane_y-pt_s(2)) )/vecn(3);

%--- coordinates of some points
pt_r(3)=pt_s(3)-((pt_r(1)-pt_s(1))*vecn(1)+ ...
                 (pt_r(2)-pt_s(2))*vecn(2)) / vecn(3); %R end point Ro
pt_o=.5*(pt_s+pt_r); %I image point Io

%=== NORMAL SHIFT ==============================
%--- the plane orthogonal to receiver ray and containing R
x=linspace(xmin+50,xmax,10);
y=linspace(ymin,ymax,10);
[plane_rx,plane_ry]=meshgrid(x,y);
 plane_rz = pt_r(3) - (vecr(1)*(plane_rx-pt_r(1))+...
                  vecr(2)*(plane_ry-pt_r(2)) )/vecr(3);

%--- the plane orthogonal to source ray and containing S
x=linspace(xmin+25,xmax-200,5);
y=linspace(ymin,ymax,5);
[plane_sx,plane_sy]=meshgrid(x,y);
 plane_sz = pt_s(3) - (vecs(1)*(plane_sx-pt_s(1))+...
                  vecs(2)*(plane_sy-pt_s(2)) )/vecs(3);
%===============================================

%----   the rays and the interface
dots = linspace(-600,600,nbpts) ;
%--- source ray
line_source_x    = pt_s(1) + vecs(1) * dots ;
line_source_y    = pt_s(2) + vecs(2) * dots ;
line_source_z    = pt_s(3) + vecs(3) * dots ;
%--- receiver ray
line_receiver_x  = pt_r(1) + vecr(1) * dots ;
line_receiver_y  = pt_r(2) + vecr(2) * dots ;
line_receiver_z  = pt_r(3) + vecr(3) * dots ;
%--- normal ray
line_normal_x    = pt_o(1) + vecn(1) * dots ;
line_normal_y    = pt_o(2) + vecn(2) * dots ;
line_normal_z    = pt_o(3) + vecn(3) * dots ;
%--- interface
[line_interface_x,line_interface_y,line_interface_z]=...
define_line(pt_s,pt_r);

%<<<<====   xOCIG for (x is constant) ====>>>>

xx  = (pt_r(1)*cos(azims)*cos(thetas)+pt_s(1)*cos(azimr)*cos(thetar))/...
      (   cos(azims)*cos(thetas)+   cos(azimr)*cos(thetar))    ;

%Coordinates of Sy, Ry and Iy
sxy = pt_s(2) + tan(azims) * (pt_s(1)-xx); %Source 
sxz = pt_s(3) + tan(thetas) * (pt_s(1)-xx) / cos(azims); %Source
pt_sx=[xx,sxy,sxz];

rxy = pt_r(2) + tan(azimr) * (pt_r(1)-xx); %Receiver
rxz = pt_r(3) + tan(thetar) * (pt_r(1)-xx) / cos(azimr); %Receiver
pt_rx=[xx,rxy,rxz];

pt_ix=.5*(pt_sx+pt_rx); %Ix middle of [Sx Rx]
%--- [ Sx Rx ] segment
[line_x_x,line_x_y,line_x_z]=define_line(pt_sx,pt_rx);

%<<<<====   yOCIG for (y is constant) ====>>>>

yy  =  (pt_r(2)*sin(azims)*cos(thetas)+pt_s(2)*sin(azimr)*cos(thetar))/...
       (   sin(azims)*cos(thetas)+   sin(azimr)*cos(thetar))    ;

%Coordinates of Sy, Ry and Iy
syx = pt_s(1) + 1/tan(azims) * (pt_s(2)-yy);
syz = pt_s(3) - tan(thetas) * (pt_s(2)-yy) / sin(azims);
pt_sy=[syx,yy,syz];

ryx = pt_r(1) + 1/tan(azimr) * (pt_r(2)-yy);
ryz = pt_r(3) - tan(thetar) * (pt_r(2)-yy) / sin(azimr);
pt_ry=[ryx,yy,ryz];

pt_iy=.5*(pt_sy+pt_ry); %Iz middle of [Sy Ry]
%--- [ Sy Ry ] segment
[line_y_x,line_y_y,line_y_z]=define_line(pt_sy,pt_ry);

%<<<<====   zOCIG (z is constant) ====>>>>

zz  =  ( pt_r(3)*sin(thetas) + pt_s(3)*sin(thetar) ) /...
       (    sin(thetas) +    sin(thetar) );

%Coordinates of Sz, Rz and Iz
szx = pt_s(1) + 1/tan(thetas) * (pt_s(3)-zz) * cos(azims);
szy = pt_s(2) - 1/tan(thetas) * (pt_s(3)-zz) * sin(azims);
pt_sz=[szx,szy,zz];

rzx = pt_r(1) + 1/tan(thetar) * (pt_r(3)-zz) * cos(azimr);
rzy = pt_r(2) - 1/tan(thetar) * (pt_r(3)-zz) * sin(azimr);
pt_rz=[rzx,rzy,zz];

%Iz middle of [Sz Rz]
pt_iz=.5*(pt_sz+pt_rz);

%--- [ Sz Rz ] segment
[line_z_x,line_z_y,line_z_z]=define_line(pt_sz,pt_rz);

%--- horizontal plane containing Iz
x=linspace(xmin-150,xmax+50,10);
y=linspace(ymin-50,ymax+50,10);
[plane_zOCIG_x,plane_zOCIG_y]=meshgrid(x,y);
 plane_zOCIG_z = ones(10)*zz ;

		 
%<<<<==== ANGLES RELATED TO THE COMMON LINE ====>>>>

%common line
%vector
vec_com = pt_iz - pt_o;
vec_com = vec_com / norm(vec_com);
%line
[line_ali_x,line_ali_y,line_ali_z]=define_line(pt_iz,pt_iy);

%direction of the line on which all the point are aligned
vec_ali = pt_ix - pt_iy ;
vec_ali = vec_ali / norm(vec_ali);
%direction of the interface
vec_int = pt_s-pt_r;
vec_int = vec_int / norm(vec_int);

%dot product to find the angle between [SR] and common line
angle_ali      = acos( vec_ali * vec_int');
angle_ali_deg  = angle_ali * 180 / pi;

%azimuth and dip of the common line
azim_ali     = acos(vec_ali(1)/sqrt(vec_ali(1)^2+vec_ali(2)^2));
azim_ali_deg = azim_ali  * 180  / pi ;
theta_ali    = acos(vec_ali(3)/sqrt(vec_ali(1)^2+vec_ali(2)^2));
theta_ali_deg= theta_ali * 180  / pi ;

%<<<<====   ORTHOGONAL PLANES TO xyzOCIG ====>>>>

%---   the plane orthogonal to xOCIG
vec_x = [ 0 , sxy-rxy , sxz-rxz ];
vec_x = vec_x / norm(vec_x);
x=linspace(xx-200,xx+200,10);
y=linspace(yy-200,yy+200,10);
[plane_x_x,plane_x_y]=meshgrid(x,y);
 plane_x_z = pt_ix(3) + (vec_x(1)*(plane_x_x-pt_ix(1) )-...
                         vec_x(2)*(plane_x_y-pt_ix(2)) )/vec_x(3);    
%--- Coeff planex
coef_plane_x=[vec_x(1),         vec_x(2),         vec_x(3),...
	      vec_x(1)*pt_ix(1)+vec_x(2)*pt_ix(2)+vec_x(3)*pt_ix(3)];

%---  plane orthogonal to yOCIG
vec_y = [ -syx+ryx  , 0 , syz-ryz ];
vec_y = vec_y / norm(vec_y);
x=linspace(xx-200,xx+200,10);
y=linspace(yy-200,yy+200,10);
[plane_y_x,plane_y_y]=meshgrid(x,y);
 plane_y_z = pt_iy(3) + (vec_y(1)*(plane_y_x-pt_iy(1))+...
                         vec_y(2)*(plane_y_y-pt_iy(2) ) )/vec_y(3);                
%--- Coeff planey
coef_plane_y = [vec_y(1),         vec_y(2),         vec_y(3),...
		vec_y(1)*pt_iy(1)+vec_y(2)*pt_iy(2)+vec_y(3)*pt_iy(3)];
                
%---  plane orthogonal to zOCIG
vec_z = [ szx-rzx , szy-rzy , 0 ];
vec_z = vec_z / norm(vec_z);
y=linspace(pt_o(2)-50,pt_o(2)+200,10);
z=linspace(pt_iz(3)-50,pt_iz(3)+200,10);
[plane_z_y,plane_z_z]=meshgrid(y,z);
 plane_z_x = pt_iz(1) + (-vec_z(2)*(plane_z_y-pt_iz(2))+...
                          vec_z(3)*(plane_z_z-pt_iz(3) ) )/vec_z(1);
%--- Coeff planez
coef_plane_z = [vec_z(1),         vec_z(2),         vec_z(3),...
		vec_z(1)*pt_iz(1)+vec_z(2)*pt_iz(2)+vec_z(3)*pt_iz(3)];

%---  intersection of planes x and y
[inter_xy_x,inter_xy_y,inter_xy_z,xxx,yyx,zzx,vec_inter_xy]=...
inter_plane_2_xy(coef_plane_y,coef_plane_x);
%---  intersection of planes x and z
[inter_xz_x,inter_xz_y,inter_xz_z,xxy,yyy,zzy,vec_inter_xz]=...
inter_plane_2_xz(coef_plane_z,coef_plane_x);
%---  intersection of planes y and z
[inter_yz_x,inter_yz_y,inter_yz_z,xxz,yyz,zzz,vec_inter_yz]=...
inter_plane_2_yz(coef_plane_y,coef_plane_z);

%dot product common line and intersection plane
vec_ali*vec_inter_xy';
%cross product of intersection plane between them
cross(vec_inter_xy,vecn);
%vec_int_xy


%--- intersection of the normal with plane orthogonal to plane x
num = dot(pt_iz-pt_o,vec_z);
den = dot(vec_z,vecn);
k   = num /den ;
pt_g = pt_o + k * vecn ;

%---   normal shift
%--- for xOCIG yOCIG zOCIG
[line_nshift_x_x,line_nshift_x_y,line_nshift_x_z]=...
define_line(pt_g,pt_ix);
[line_nshift_y_x,line_nshift_y_y,line_nshift_y_z]=...
define_line(pt_g,pt_iy);
[line_nshift_z_x,line_nshift_z_y,line_nshift_z_z]=...
define_line(pt_g,pt_iz);

[azim,theta]=vec2angle(vec_ali);
[azim,theta]=vec2angle(vec_inter_xy);

%|||||||||||||||||||||||||||||||||||||
%|||||||||||||||||||||||||||||||||||||
%<<<<======= NEW  GEOMETRY =======>>>>
%|||||||||||||||||||||||||||||||||||||
%|||||||||||||||||||||||||||||||||||||

%<<<<==== WHAT IS THE ROTATION TO MAKE THE RAYS COPLANAR ?

%--- Normal of the coplanar plane
vec_n = cross(vecs,vecr)';
vec_n = vec_n / norm(vec_n);
%--- Plane of coplanarity:
%--- Plane containing Io and with vec_n as normal
x=linspace(xmin,xmax,5);
y=linspace(ymin,ymax,5);
[plane_x_copl,plane_y_copl]=meshgrid(x,y);
 plane_z_copl = pt_o(3)-(vec_n(1)*(plane_x_copl-pt_o(1))+...
                         vec_n(2)*(plane_y_copl-pt_o(2)))/vec_n(3);

%========================================================
%=== Rotation in the interface ==========================
%========================================================
%--- angle of the rotation to make the rays coplanar
vec_so_io = pt_s-pt_o;
vec_so_io = vec_so_io / norm(vec_so_io);
ang_rot   = acos(dot(vec_so_io,vec_com))*180/pi;
%--- coordinates of S2 and R2 on the coplanar plane
%--- radius: IS or IR
radius = norm(pt_o-pt_s);
pt_s2 = pt_o + radius * vec_com ;
pt_r2 = pt_o - radius * vec_com ;
%--- S2-R2 segment
[line_interface_copl_x,line_interface_copl_y,line_interface_copl_z]=...
define_line(pt_s2,pt_r2);
%--- New source and receiver rays
%--- New source ray
line_source_copl_x   = pt_s2(1) + vecs(1) * dots ;
line_source_copl_y   = pt_s2(2) + vecs(2) * dots ;
line_source_copl_z   = pt_s2(3) + vecs(3) * dots ;
%--- New receiver ray
line_receiver_copl_x = pt_r2(1) + vecr(1) * dots ;
line_receiver_copl_y = pt_r2(2) + vecr(2) * dots ;
line_receiver_copl_z = pt_r2(3) + vecr(3) * dots ;

%--- Intersection of the two coplanar rays
rad=200;
%intermediate points
pt_s_tmp = pt_s2 + rad * vecs ;
pt_r_tmp = pt_r2 + rad * vecr ;
%Thales
rad=-abs(norm(pt_s2   -pt_s_tmp)*norm(pt_r2-pt_s2)/...
        (norm(pt_r_tmp-pt_s_tmp)-norm(pt_s2-pt_r2)));
pt_cross = pt_s2 + rad*vecs ;

%========================================================
%=== Rotation in the zOCIG ==============================
%========================================================
% geological-dip coordinates basis
[vec_ngx,junk,junk]=geological_dip_basis(vec_n,pt_iz,0);
%---new Sz2 ans Rz2
radi=norm(pt_iz-pt_sz);
pt_sz_2=pt_iz+radi*vec_ngx;
pt_rz_2=pt_iz-radi*vec_ngx;
vec_int_2=(pt_sz_2-pt_rz_2)/norm(pt_sz_2-pt_rz_2);
%---Sz2-Rz2 segment
[line_zocig_copl_x,line_zocig_copl_y,line_zocig_copl_z]=...
define_line(pt_sz_2,pt_rz_2);
%---New source and receiver rays
line_source_copl2_x   = pt_sz_2(1) + vecs(1) * dots ;
line_source_copl2_y   = pt_sz_2(2) + vecs(2) * dots ;
line_source_copl2_z   = pt_sz_2(3) + vecs(3) * dots ;
line_receiver_copl2_x = pt_rz_2(1) + vecr(1) * dots ;
line_receiver_copl2_y = pt_rz_2(2) + vecr(2) * dots ;
line_receiver_copl2_z = pt_rz_2(3) + vecr(3) * dots ;
%---Intersection of the two coplanar rays
%intermediate points
j=zz+5;
tmp=(j-zz)/vecs(3);
pt_s_tmp=pt_sz_2+tmp*vecs;
tmp=(j-zz)/vecr(3);
pt_r_tmp=pt_rz_2+tmp*vecr;
%Thales
rad=-abs(norm(pt_sz_2 -pt_s_tmp)*norm(pt_rz_2-pt_sz_2)/...
        (norm(pt_r_tmp-pt_s_tmp)-norm(pt_sz_2-pt_rz_2)));
pt_cross_2 = pt_sz_2 + rad*vecs ;
%angle of the rotation in zOCIG plane
ang_rot = acos(dot(vec_int,vec_int_2))*180/pi

%|||||||||||||||||||||||||||||||||||||
%|||||||||||||||||||||||||||||||||||||
%<<<<==== PLOTTING EVERYTHING ====>>>>
%|||||||||||||||||||||||||||||||||||||
%|||||||||||||||||||||||||||||||||||||

colormap(gray)
shading interp
axis vis3d off
%--- Display
xmax=max([xx,rzy,szy,pt_r(1),pt_s(1)])+200;
xmin=min([xx,rzy,szy,pt_r(1),pt_s(1)])-200;
ymax=max([yy,rzy,szy,pt_r(2),pt_s(2)])+200;
ymin=min([yy,rzy,szy,pt_r(2),pt_s(2)])-200;
zmax=max([zz,rxz,sxz,pt_r(3),pt_s(3)])+100;
zmin=min([zz,rxz,sxz,pt_r(3),pt_s(3)])-100;
%axis([700 1600 700 1200 800 1500])
axis([xmin xmax ymin ymax zmin zmax])
axis equal
grid
xlabel('x')
ylabel('y')
zlabel('z')
% POINT OF VIEW - CAMERA POSITION
campos([-3251.6 ,  -3137.8 ,   4840.5])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GIVEN GEOMETRY OF THE PROBLEM	||||||||||||||||||||||||%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(1)							%
%source and receiver end point S and R and image point	%
plot3(pt_s(1),pt_s(2),pt_s(3),'r*') %So			%
plot3(pt_r(1),pt_r(2),pt_r(3),'bv') %Ro			%
plot3(pt_o(1),pt_o(2),pt_o(3),'ko') %Io			%
%[SR] segment						%
plot3(line_interface_x,line_interface_y,line_interface_z,'k')
%source receiver and 
plot3(line_source_x  ,line_source_y  ,line_source_z  ,'r')
plot3(line_receiver_x,line_receiver_y,line_receiver_z,'b')
end
if(1)
%normal ray
plot3(line_normal_x  ,line_normal_y  ,line_normal_z  ,'k-.')
%interface						%
surf(plane_x,plane_y,plane_z)				%
end							%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% OFFSETS GATHERS ||||||||||||||||||||||||||||||||||||||%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(0)							%
%=== xOCIG						%
plot3(xx,sxy,sxz,'c*') %Sx				%
plot3(xx,rxy,rxz,'cv') %Rx				%
plot3(xx,ixy,ixz,'co') %Ix				%
%[Sx Rx] segment					%
plot3(line_x_x ,line_x_y ,line_x_z ,'c-') 		%
%=== yOCIG						%
plot3(syx,yy,syz,'m*') %Sy				%
plot3(ryx,yy,ryz,'mv') %Ry				%
plot3(iyx,yy,iyz,'mo') %Iy				%
%[Sy Ry] segment					%
plot3(line_y_x ,line_y_y ,line_y_z ,'m-')		%
end							%
if(0)							%
%=== zOCIG						%
plot3(szx,szy,zz,'r*') %Sz				%
plot3(rzx,rzy,zz,'bv') %Rz				%
plot3(pt_iz(1),pt_iz(2),pt_iz(3),'go') %Iz		%
%[Sxh Rxh] segment					%
plot3(line_z_x,line_z_y,line_z_z,'g--')			%
end							%
%plot3(line_ali_x,line_ali_y,line_ali_z,'k-')		%
if(0)							%
% plan ortho to [Sx Rx] [Sy Ry] [Sz Rz]			%
surf(plane_x_x,plane_x_y,plane_x_z) 			%
surf(plane_y_x,plane_y_y,plane_y_z) 			%
surf(plane_z_x,plane_z_y,plane_z_z) 			%
end							%
% horizontal plane including Iz				%
%surf(plane_zOCIG_x,plane_zOCIG_y,plane_zOCIG_z)		%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GAMMA - NORMAL SHIFT |||||||||||||||||||||||||||||||||%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%=== normal shift					%
if(1)							%
% plan ortho to rec ray and containing R		%
%surf(plane_rx,plane_ry,plane_rz)			%
% plan ortho to sou ray and containing S		%
%surf(plane_sx,plane_sy,plane_sz)			%
%--- coplanar plane					%
%surf(plane_x_copl,plane_y_copl,plane_z_copl) 		%
end							%
%=== alignment of points				%
if(0)							%
plot3(pt_iz(1),pt_iz(2),pt_iz(3),'go') %Iz		%
plot3(pt_iy(1),pt_iy(2),pt_iy(3),'mo') %Iy		%
plot3(pt_ix(1),pt_ix(2),pt_ix(3),'co') %Ix		%
plot3(line_ali_x,line_ali_y,line_ali_z,'k-')		%
end							%
if(0)							%
%plot3(line_z_x ,line_z_y ,line_z_z ,'g-')		%
%plot3(line_y_x ,line_y_y ,line_y_z ,'m-')		%
%plot3(line_x_x ,line_x_y ,line_x_z ,'c-') 		%
%=== normal shift					%
%[Ix Ig] [Iy Ig] [Iz Ig] segment			%
plot3(line_nshift_x_x,line_nshift_x_y,line_nshift_x_z,'c-') 	
plot3(line_nshift_y_x,line_nshift_y_y,line_nshift_y_z,'m-') 	
plot3(line_nshift_z_x,line_nshift_z_y,line_nshift_z_z,'g-')
end							%
%=== gamma						%
%plot3(pt_g(1),pt_g(2),pt_g(3),'pk')			%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COPLANARITY ||||||||||||||||||||||||||||||||||||||||||%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%--- coplanar plane					%
%surf(plane_x_copl,plane_y_copl,plane_z_copl) 		%
if(1)							%
%=== Rotation in the interface				%
%--- source S2 and receiver R2 on the coplanar plane	%
plot3(pt_s2(1),pt_s2(2),pt_s2(3),'*r')			%
plot3(pt_r2(1),pt_r2(2),pt_r2(3),'vb')			%
%--- crossing point					%
plot3(pt_cross(1),pt_cross(2),pt_cross(3),'ko')		%
%--- S2-R2 segment					%
plot3(line_interface_copl_x,line_interface_copl_y,...	%
      line_interface_copl_z,'k--')			%
%--- coplanar source and receiver rays			%
plot3(line_source_copl_x   ,line_source_copl_y   ,...	%
      line_source_copl_z   ,'r--')		 	%
plot3(line_receiver_copl_x ,line_receiver_copl_y ,...	%
      line_receiver_copl_z ,'b--')			%
end							%
if(0)							%
%=== Rotation in the zOCIG				%
%--- source S2 and receiver R2 on the coplanar plane	%
plot3(pt_sz_2(1),pt_sz_2(2),pt_sz_2(3),'r*')		%
plot3(pt_rz_2(1),pt_rz_2(2),pt_rz_2(3),'bv')		%
%--- crossing point					%
plot3(pt_cross_2(1),pt_cross_2(2),pt_cross_2(3),'ko')	%
%---Sz2-Rz2 segment					%
plot3(line_zocig_copl_x,line_zocig_copl_y,line_zocig_copl_z,'k--')
%--- coplanar source and receiver rays			%
plot3(line_source_copl2_x,line_source_copl2_y,...	%
      line_source_copl2_z,'r--')			%
plot3(line_receiver_copl2_x,line_receiver_copl2_y,...	%
      line_receiver_copl2_z,'b--')			%
end							%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 	 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ANIMATION WITH ORIGINAL AND COPLANAR RAYS ||||||||||||%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(0)							%
axis vis3d off						%
%for x = 5000:-100:-3251.6				%
%  campos([x ,  -3137.8 ,   4840.5])			%
%  drawnow						%
%end							%
for x = 3*pi/2:.01:6*pi/2				%
  campos([3000*cos(x) ,  3000*sin(x) ,  5000 ])		%
  drawnow						%
end							%
end							%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PROJECTIONS IN THE INTERFACE |||||||||||||||||||||||||%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(0)
%===
radius=450;
%coordinates of new point on S ray
pt_s3 = pt_s + radius * vecs;
plot3(pt_s3(1),pt_s3(2),pt_s3(3),'go')
vec_ss=pt_s3-pt_s;
%projection on normal
vec_proj_nor_ss =  dot(vec_ss,vecn)*vecn;
pt_s5 = pt_s + vec_proj_nor_ss;
%projection on interface
vec_sh=pt_s3-pt_s5;
pt_sh=pt_s+vec_sh;
plot3(pt_sh(1),pt_sh(2),pt_sh(3),'go');
%line
[line_s_ssh_x,line_s_ssh_y,line_s_ssh_z]=...
define_line(pt_s,pt_sh);
[line_s_sssh_x,line_s_sssh_y,line_s_sssh_z]=...
define_line(pt_s3,pt_sh);
plot3(line_s_ssh_x,line_s_ssh_y,line_s_ssh_z,'g')
plot3(line_s_sssh_x,line_s_sssh_y,line_s_sssh_z,'g')
%===
%coordinates of new point on R ray
pt_r3=pt_r+radius*vecr;
plot3(pt_r3(1),pt_r3(2),pt_r3(3),'go')
vec_rr=pt_r3-pt_r;
%projection on normal
vec_proj_nor_rr =  dot(vec_rr,vecn)*vecn;
pt_r5=pt_r+vec_proj_nor_rr;
%projection on interface
vec_rh=pt_r3-pt_r5;
pt_rh = pt_r + vec_rh;
plot3(pt_rh(1),pt_rh(2),pt_rh(3),'go');
%line
[line_r_rrh_x,line_r_rrh_y,line_r_rrh_z]=...
define_line(pt_r,pt_rh);
[line_r_rrrh_x,line_r_rrrh_y,line_r_rrrh_z]=...
define_line(pt_r3,pt_rh);
plot3(line_r_rrh_x,line_r_rrh_y,line_r_rrh_z,'g')
plot3(line_r_rrrh_x,line_r_rrrh_y,line_r_rrrh_z,'g')
%===
% segment: [SSH RRH]
[line_ssh_rrh_x,line_ssh_rrh_y,line_ssh_rrh_z]=...
define_line(pt_sh,pt_rh);
% geological-dip coordinates basis (Shan's proposal)	%
geological_dip_basis(vecn,pt_o,0);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PROJECTIONS ON THE zOCIG |||||||||||||||||||||||||||||%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(0)
%PROJECTION OF THE ORIGINAL RAYS
%=== 
radius=450;
%coordinates of new point on S ray
pt_s3 = pt_sz + radius * vecs;
vec_ss=pt_s3-pt_sz;
%projection on vertical
vec_proj_nor_ss =  dot(vec_ss,[0 0 1])*[0 0 1];
pt_s5 = pt_sz + vec_proj_nor_ss;
%projection on zOCIG
vec_sh=pt_s3-pt_s5;
pt_sh=pt_sz+vec_sh;
%line
[line_s_ssh_x,line_s_ssh_y,line_s_ssh_z]=...
define_line(pt_sz,pt_sh);
[line_s_sssh_x,line_s_sssh_y,line_s_sssh_z]=...
define_line(pt_s3,pt_sh);
%===
%coordinates of new point on R ray
pt_r3=pt_rz+radius*vecr;
vec_rr=pt_r3-pt_rz;
%projection on vertical
vec_proj_nor_rr =  dot(vec_rr,[0 0 1])*[0 0 1];
pt_r5=pt_rz+vec_proj_nor_rr;
%projection on interface
vec_rh=pt_r3-pt_r5;
pt_rh = pt_rz + vec_rh;
%line
[line_r_rrh_x,line_r_rrh_y,line_r_rrh_z]=...
define_line(pt_rz,pt_rh);
[line_r_rrrh_x,line_r_rrrh_y,line_r_rrrh_z]=...
define_line(pt_r3,pt_rh);
if(1)
%plot3(pt_s3(1),pt_s3(2),pt_s3(3),'go')
%plot3(pt_sh(1),pt_sh(2),pt_sh(3),'go');
plot3(line_s_ssh_x,line_s_ssh_y,line_s_ssh_z,'g')
plot3(line_s_sssh_x,line_s_sssh_y,line_s_sssh_z,'g')
%plot3(pt_r3(1),pt_r3(2),pt_r3(3),'go')
%plot3(pt_rh(1),pt_rh(2),pt_rh(3),'go');
plot3(line_r_rrh_x,line_r_rrh_y,line_r_rrh_z,'g')
plot3(line_r_rrrh_x,line_r_rrrh_y,line_r_rrrh_z,'g')
end

%PROJECTION OF THE ROTATED RAYS
%=== 
radius=450;
%coordinates of Sz2 of rotated ray on zOCIG
pt_s3=pt_sz_2+(pt_sz_2-pt_cross_2);
%pt_s3 = pt_sz_2 + radius * vecs;
vec_ss=pt_s3-pt_sz_2;
%projection on vertical
vec_proj_nor_ss =  dot(vec_ss,[0 0 1])*[0 0 1];
pt_s5 = pt_sz_2 + vec_proj_nor_ss;
%projection on zOCIG
vec_sh=pt_s3-pt_s5;
pt_sh=pt_sz_2+vec_sh;
%line
[line_s_ssh_x,line_s_ssh_y,line_s_ssh_z]=...
define_line(pt_sz_2,pt_sh);
[line_s_sssh_x,line_s_sssh_y,line_s_sssh_z]=...
define_line(pt_s3,pt_sh);
%===
%coordinates of new point on R ray
pt_r3=pt_rz_2+(pt_rz_2-pt_cross_2);
vec_rr=pt_r3-pt_rz_2;
%projection on vertical
vec_proj_nor_rr =  dot(vec_rr,[0 0 1])*[0 0 1];
pt_r5=pt_rz_2+vec_proj_nor_rr;
%projection on interface
vec_rh=pt_r3-pt_r5;
pt_rh = pt_rz_2 + vec_rh;
%line
[line_r_rrh_x,line_r_rrh_y,line_r_rrh_z]=...
define_line(pt_rz_2,pt_rh);
[line_r_rrrh_x,line_r_rrrh_y,line_r_rrrh_z]=...
define_line(pt_r3,pt_rh);
%===
%projection of intersection point
pt_cross_2_h=[pt_cross_2(1) pt_cross_2(2) zz];
[line_cross2sx,line_cross2sy,line_cross2sz]=...
define_line(pt_sz_2,pt_cross_2_h);
[line_cross2rx,line_cross2ry,line_cross2rz]=...
define_line(pt_rz_2,pt_cross_2_h);
[line_cross_projx,line_cross_projy,line_cross_projz]=...
define_line(pt_cross_2,pt_cross_2_h);
if(1)
%plot3(pt_s3(1),pt_s3(2),pt_s3(3),'go')
%plot3(pt_sh(1),pt_sh(2),pt_sh(3),'go');
plot3(line_s_ssh_x,line_s_ssh_y,line_s_ssh_z,'g')
plot3(line_s_sssh_x,line_s_sssh_y,line_s_sssh_z,'g')
%plot3(pt_r3(1),pt_r3(2),pt_r3(3),'go')
%plot3(pt_rh(1),pt_rh(2),pt_rh(3),'go');
plot3(line_r_rrh_x,line_r_rrh_y,line_r_rrh_z,'g')
plot3(line_r_rrrh_x,line_r_rrrh_y,line_r_rrrh_z,'g')
plot3(pt_cross_2_h(1),pt_cross_2_h(2),pt_cross_2_h(3),'go')
plot3(line_cross2sx,line_cross2sy,line_cross2sz,'g-.')
plot3(line_cross2rx,line_cross2ry,line_cross2rz,'g-.')
plot3(line_cross_projx,line_cross_projy,line_cross_projz,'g--')
end


end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

shading interp


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%===========================
%^^^^^^^^^^^^^^^^^^^^^^^^^^^

%=========================================================
function [line_x,line_y,line_z]=define_line(pt_beg,pt_end)
%--- defines a segment between two given points
nbpts=10;
line_x = linspace(pt_beg(1),pt_end(1),nbpts) ;
line_y = linspace(pt_beg(2),pt_end(2),nbpts) ;
line_z = linspace(pt_beg(3),pt_end(3),nbpts) ;

%=========================================================
function [theta,azim]=vec2angle(vec)
%given the coordinates of a vector, the function
%returns the azimuth and dip of the vector in degrees
%WARNING: doesnt work if input vector = [0 0 1]
vec = vec / norm(vec);
d = sqrt( vec(1)^2 + vec(2)^2 );
if(d==0)
warning off MATLAB:divideByZero
end
theta = atan( vec(3) / d ) * 180 / pi ;
azim  = acos( vec(1) / d ) * 180 / pi ;
if(d==0)
disp('#### azimuth not defined for such vector ####')
end

%=========================================================
function [vec]=angle2vec(theta,azim)
%given the 2 angles of a vector, the function
%returns the coefficients of the vector
vec(1)=-cos(theta)*cos(azim);
vec(2)=cos(theta)*sin(azim);
vec(3)=sin(theta);

%=========================================================
function [vec_ngx,vec_ngy,vec_ngz]=geological_dip_basis(vec_n,pt_center,basis)
%defines the geological dip basis of the plane defined by
%its normal (cf Shan's proposal 2003)
[theta_n,phi_n]=vec2angle(vec_n);
phi_n   = phi_n   / 180 * pi ;
theta_n = theta_n / 180 * pi ;
vec_ngx = [-sin(  phi_n) cos(phi_n) 0   ];
vec_ngy = [ sin(theta_n)*cos(phi_n) ...
            sin(theta_n)*sin(phi_n) ...
	   -cos(theta_n)		];
vec_ngz = [ cos(theta_n)*cos(phi_n) ...
            cos(theta_n)*sin(phi_n) ...
	    sin(theta_n)		];
basis
if(basis==1)
%--- draw basis
radius=300;
pt_ngx = pt_center + radius * vec_ngx ;
pt_ngy = pt_center + radius * vec_ngy ;
pt_ngz = pt_center + radius * vec_ngz ;
[line_ngx_x,line_ngx_y,line_ngx_z]=...
define_line(pt_center,pt_ngx);
plot3(line_ngx_x,line_ngx_y,line_ngx_z,'m')
[line_ngy_x,line_ngy_y,line_ngy_z]=...
define_line(pt_center,pt_ngy);
plot3(line_ngy_x,line_ngy_y,line_ngy_z,'m')
[line_ngz_x,line_ngz_y,line_ngz_z]=...
define_line(pt_center,pt_ngz);
plot3(line_ngz_x,line_ngz_y,line_ngz_z,'m')
end

%vvvvvvvvvvvvvvvvvvvvvvvvvvv
%===========================
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

