function hvocig3Dter(xs,ys,zs,xr,yr,thetas,azims,thetar,azimr)

%(xs,ys,zs) coordinates of the end point of source   ray
%(xr,yr,zr) coordinates of the end point of receiver ray
%thetas,azims: dippnig and azimuth of source ray at endind point
%thetar,azimr: dippnig and azimuth of receiver ray at endind point

%examples of values (1000,1000,1000,1500,900,20,30,60,80)

nbpts = 20 ;    %number of points to draw lines

%conversion in radian of alpha and gamma angles
thetas = thetas * pi / 180 ;
thetar = thetar * pi / 180 ;
azimr  = azimr  * pi / 180 ;
azims  = azims  * pi / 180 ;


%<<<<==== COMPUTING SOURCE AND RECEIVER RAY ====>>>>


%---- ray vectors for source, receiver, normal
vecs = [ -cos(thetas)*cos(azims)...
          cos(thetas)*sin(azims)...
          sin(thetas) ] ;
vecr = [ -cos(thetar)*cos(azimr)...
          cos(thetar)*sin(azimr)...
          sin(thetar) ] ;

vecn = vecs + vecr ; %normal ray direction
vecn = vecn / norm(vecn) ; %normalization

%--- Window
xmax=max(xr,xs)+100;
ymax=max(yr,ys)+100;
xmin=min(xr,xs)-100;
ymin=min(yr,ys)-100;

%--- Computation of the plane orthogonal to normal and containing S
x=linspace(xmin,xmax,5);
y=linspace(ymin,ymax,5);
[plane_x,plane_y]=meshgrid(x,y);
 plane_z = zs - (vecn(1)*(plane_x-xs)+...
                 vecn(2)*(plane_y-ys) )/vecn(3);

%---- coordinates of some points
zr = zs - ( ( xr - xs ) * vecn(1) + ...
            ( yr - ys ) * vecn(2)      ) / vecn(3); %receiver end point Ro
xo = ( xs + xr ) / 2 ; % middle of SoRo: Io
yo = ( ys + yr ) / 2 ; % middle of SoRo: Io
zo = ( zs + zr ) / 2 ; % middle of SoRo: Io

%---- computing the rays and the interface
dots = linspace(-600,600,nbpts) ;
%--- source ray
line_source_x    = xs + vecs(1) * dots ;
line_source_y    = ys + vecs(2) * dots ;
line_source_z    = zs + vecs(3) * dots ;
%--- receiver ray
line_receiver_x  = xr + vecr(1) * dots ;
line_receiver_y  = yr + vecr(2) * dots ;
line_receiver_z  = zr + vecr(3) * dots ;
%--- normal ray
line_normal_x    = xo + vecn(1) * dots ;
line_normal_y    = yo + vecn(2) * dots ;
line_normal_z    = zo + vecn(3) * dots ;
%--- interface
line_interface_x = xs + linspace(0,xr-xs,nbpts) ;
line_interface_y = ys + linspace(0,yr-ys,nbpts) ;
line_interface_z = zs + linspace(0,zr-zs,nbpts) ;


%<<<<==== COMPUTING VxOCIG for (x is constant) ====>>>>


xx  =  (xr*cos(azims)*cos(thetas)+xs*cos(azimr)*cos(thetar))/...
       (   cos(azims)*cos(thetas)+   cos(azimr)*cos(thetar))    ;

sxy = ys + tan(azims) * (xs-xx); %Source 
rxy = yr + tan(azimr) * (xr-xx); %Receiver
ixy = ( sxy + rxy ) / 2 ;        %Ix middle of [Sx Rx]

sxz = zs + tan(thetas) * (xs-xx) / cos(azims); %Source
rxz = zr + tan(thetar) * (xr-xx) / cos(azimr); %Receiver
ixz = ( sxz + rxz ) / 2 ; %Ix middle of [Sx Rx]

%--- [ Sx Rx ] segment
line_x_x = xx + zeros(nbpts/4) ;
line_x_y = linspace(sxy,rxy,nbpts/4) ;
line_x_z = linspace(sxz,rxz,nbpts/4) ;

%compute the line orthogonal to the VxOCIG passing through Ix
line_gx_x = linspace(xs,xx,nbpts);
line_gx_y = zeros(nbpts) + ixy ;
line_gx_z = zeros(nbpts) + ixz ;


%<<<<==== COMPUTING VyOCIG for (y is constant) ====>>>>


yy  =  (yr*sin(azims)*cos(thetas)+ys*sin(azimr)*cos(thetar))/...
       (   sin(azims)*cos(thetas)+   sin(azimr)*cos(thetar))    ;

syz = zs - tan(thetas) * (ys-yy) / sin(azims);
ryz = zr - tan(thetar) * (yr-yy) / sin(azimr);
iyz = ( syz + ryz ) / 2;  %Iy middle of [Sy Ry]

syx = xs + 1/tan(azims) * (ys-yy);
ryx = xr + 1/tan(azimr) * (yr-yy);
iyx = ( syx + ryx ) / 2;  %Iy middle of [Sy Ry]

%--- [ Sy Ry ] segment
line_y_x = linspace(syx,ryx,nbpts/4) ;
line_y_y = yy + zeros(nbpts/4) ;
line_y_z = linspace(syz,ryz,nbpts/4) ;

%compute the line orthogonal to the VyOCIG passing through Iy
line_gy_x = zeros(nbpts) + iyx ;
line_gy_y = linspace(yy,sxy,nbpts);
line_gy_z = zeros(nbpts) + iyz ;


%<<<<==== COMPUTING HOCIG (z is constant) ====>>>>


zz  =  ( zr*sin(thetas) + zs*sin(thetar) ) /...
       (    sin(thetas) +    sin(thetar) )    ;

%Coordinates of Sz, Rz and Iz
szx = xs + 1/tan(thetas) * (zs-zz) * cos(azims);
rzx = xr + 1/tan(thetar) * (zr-zz) * cos(azimr);
izx = ( szx + rzx ) / 2 ; %Iz middle of [Sz Rz]

szy = ys - 1/tan(thetas) * (zs-zz) * sin(azims);
rzy = yr - 1/tan(thetar) * (zr-zz) * sin(azimr);
izy = ( szy + rzy ) / 2 ; %Iz middle of [Sz Rz]

%--- [ Sz Rz ] segment
line_z_x = linspace(szx,rzx,nbpts) ;
line_z_y = linspace(szy,rzy,nbpts) ;
line_z_z = zz + zeros(nbpts) ;

%compute the line orthogonal to the HOCIG passing through Iz
line_gz_x = zeros(nbpts) + izx ;
line_gz_y = zeros(nbpts) + izy ;
line_gz_z = linspace(zz,ixz,nbpts);

%common line
line_ali_x = linspace(izx,xx ,nbpts);
line_ali_y = linspace(izy,ixy,nbpts);
line_ali_z = linspace(zz ,ixz,nbpts);


%<<<<==== ANGLES RELATED TO THE COMMON LINE====>>>>


%direction of the line on which all the point are aligned
vec_ali = [ xx - iyx , ixy - yy , ixz - iyz ];
vec_ali = vec_ali / norm(vec_ali);
%direction of the interface
vec_int = [ xs - xr  , ys  - yr , ys  - zr  ];
vec_int = vec_int / norm(vec_int);

%dot product to find the angle between [SR] and common line
angle_ali      = acos( vec_ali * vec_int');
angle_ali_deg  = angle_ali * 180 / pi

%azimuth and dipping of the common line
azim_ali     = acos(vec_ali(1)/sqrt(vec_ali(1)^2+vec_ali(2)^2));
azim_ali_deg = azim_ali  * 180  / pi 
theta_ali    = acos(vec_ali(3)/sqrt(vec_ali(1)^2+vec_ali(2)^2));
theta_ali_deg= theta_ali * 180  / pi 


%<<<<==== COMPUTING ORTHOGONAL PLANES TO HOCIG and VOGIG ====>>>>


%--- Computation of the plane orthogonal to VxOCIG
vec_x = [ 0 , sxy-rxy , sxz-rxz ];
vec_x = vec_x / norm(vec_x);
x=linspace(800,1600,5);
y=linspace(600,1200,5);
[plane_x_x,plane_x_y]=meshgrid(x,y);
 plane_x_z = ixz + (vec_x(1)*(plane_x_x-xx )-...
                    vec_x(2)*(plane_x_y-ixy) )/vec_x(3);    
%--- Coeff planex
coef_plane_x = [vec_x(1),     vec_x(2),      vec_x(3),...
		vec_x(1)*xx + vec_x(2)*ixy + vec_x(3)*ixz];

%--- Computation of the plane orthogonal to VyOCIG
vec_y = [ -syx+ryx  , 0 , syz-ryz ];
vec_y = vec_y / norm(vec_y);
x=linspace(800,1600,5);
y=linspace(600,1200,5);
[plane_y_x,plane_y_y]=meshgrid(x,y);
 plane_y_z = iyz + (vec_y(1)*(plane_y_x-iyx)+...
                    vec_y(2)*(plane_y_y-yy ) )/vec_y(3);                
%--- Coeff planey
coef_plane_y = [vec_y(1),      vec_y(2),     vec_y(3),...
		vec_y(1)*iyx + vec_y(2)*yy + vec_y(3)*iyz];
                
%--- Computation of the plane orthogonal to HOCIG
vec_z = [ szx-rzx , szy-rzy , 0 ];
vec_z = vec_z / norm(vec_z);
y=linspace(600,1200,5);
z=linspace(1100,1500,5);
[plane_z_y,plane_z_z]=meshgrid(y,z);
 plane_z_x = izx + (-vec_z(2)*(plane_z_y-izy)+...
                    vec_z(3)*(plane_z_z- zz) )/vec_z(1);
%--- Coeff planez
coef_plane_z = [vec_z(1),      vec_z(2),      vec_z(3),...
		vec_z(1)*izx + vec_z(2)*izy + vec_z(3)*zz];

%--- Computation of the intersection of planes x and y
[inter_xy_x,inter_xy_y,inter_xy_z]=...
inter_plane(coef_plane_x,coef_plane_y);
bonjour=1
%--- Computation of the intersection of planes x and z
[inter_xz_x,inter_xz_y,inter_xz_z]=...
inter_plane(coef_plane_x,coef_plane_z);
bonjour=2
%--- Computation of the intersection of planes y and z
[inter_yz_x,inter_yz_y,inter_yz_z]=...
inter_plane(coef_plane_y,coef_plane_z);
bonjour=3


%<<<<==== PLOTTING EVERYTHING ====>>>>


clf
hold on
%--- given geometry of the problem
%plot3(xs,ys,zs,'r*') %So
%plot3(xr,yr,zr,'bv') %Ro
%plot3(xo,yo,zo,'ko') %Io
%plot3(line_interface_x,line_interface_y,line_interface_z,'k') %interface
%mesh(plane_x,plane_y,plane_z)
%plot3(line_normal_x  ,line_normal_y  ,line_normal_z  ,'k') %normal
%plot3(line_source_x  ,line_source_y  ,line_source_z  ,'r') %source ray
%plot3(line_receiver_x,line_receiver_y,line_receiver_z,'b') %receiver ray

%--- VOCIG - x 
%plot3(xx,sxy,sxz,'c*') %Sx
%plot3(xx,rxy,rxz,'cv') %Rx
%plot3(xx,ixy,ixz,'co') %Ix
%plot3(line_x_x ,line_x_y ,line_x_z ,'c-') 	%[ Sx Rx ] segment
%plot3(line_gx_x,line_gx_y,line_gx_z,'c-') 	%[ Ix Ig ] segment
mesh(plane_x_x,plane_x_y,plane_x_z) % plan ortho to [ Sx Rx ]

%--- VOCIG - y
%plot3(syx,yy,syz,'m*') %Sy
%plot3(ryx,yy,ryz,'mv') %Ry
%plot3(iyx,yy,iyz,'mo') %Iy
%plot3(line_y_x ,line_y_y ,line_y_z ,'m-') 	%[Szv2 Rzv2] segment
%plot3(line_gy_x,line_gy_y,line_gy_z,'m-') 	%[ Ix Ig ] segment
mesh(plane_y_x,plane_y_y,plane_y_z) % plan ortho to [ Sy Ry ]

%--- HOCIG - z
%plot3(szx,szy,zz,'g*') %Sz
%plot3(rzx,rzy,zz,'gv') %Rz
%plot3(izx,izy,zz,'go') %Iz
%plot3(line_z_x,line_z_y,line_z_z,'g-') 		%[ Sxh Rxh] segment
%plot3(line_gz_x,line_gz_y,line_gz_z,'g') 	%[ Ihx Ig ] segment
mesh(plane_z_x,plane_z_y,plane_z_z) % plan ortho to [ Sz Rz ]

%plot3(xx,yy,zz,'kp')
%plot3(line_ali_x,line_ali_y,line_ali_z,'k') 	%alignment of points

plot3(inter_xy_x,inter_xy_y,inter_xy_z,'c')
plot3(inter_xz_x,inter_xz_y,inter_xz_z,'m')
plot3(inter_yz_x,inter_yz_y,inter_yz_z,'g')

%--- Display
%xmax=max(max(xx,max(rzy,szy)),max(xr,xs))+200;
%xmin=min(min(xx,min(xrh,xsh)),min(xr,xs))-150;
%zmax=max(max(zz,max(rxz,sxz)),max(zr,zs))+200;
%zmin=min(min(zz,min(rxz,sxz)),min(zr,zs))-150;
axis([700 1600 700 1200 700 2000])
axis equal
grid
xlabel('x')
ylabel('y')
zlabel('z')
title('3D HOCIG and VOCIG with wrong velocity ')
