function pt_g=image_position(xs,ys,zs,xr,yr,...
                     thetas,azims,thetar,azimr)

%(pt_s(1),pt_s(2),pt_s(3)) coordinates of the end point of source   ray
%(pt_r(1),pt_r(2),pt_r(3)) coordinates of the end point of receiver ray
%thetas,azims: dip and azimuth of source ray at end point
%thetar,azimr: dip and azimuth of receiver ray at end point

%examples of values (1000,1000,1000,1500,900,20,30,60,80)
%examples of values image_position(1000,1200,1000,1250,900,25,20,55,70)


pt_sz=[xs ys zs];
pt_rz=[xr yr zs];

nbpts = 20 ;    %number of points to draw lines

%conversion in radian of alpha and gamma angles
thetas = thetas * pi / 180 ;
thetar = thetar * pi / 180 ;
azimr  = azimr  * pi / 180 ; 
azims  = azims  * pi / 180 ;

%|||||||||||||||||||||||||||||||||||||
%<<<<===== ORIGINAL GEOMETRY =====>>>>
%|||||||||||||||||||||||||||||||||||||

%<<<<==== SOURCE AND RECEIVER RAY ====>>>>

%---- ray vectors for source, receiver, normal
vecs=angle2vec(thetas,azims);
vecr=angle2vec(thetar,azimr);

vecn = vecs + vecr ; %normal ray direction
vecn = vecn / norm(vecn) ; %normalization

%#########################################
%<<<<==== zOCIG (z is constant) ====>>>>%#
num=dot(pt_sz-pt_rz,vecn);		%#
den=dot(vecs,vecn)+dot(vecr,vecn);	%#
d=num/den;				%#
pt_s=pt_sz-d*vecs;			%#
pt_r=pt_rz+d*vecr;			%#
dot(pt_s-pt_r,vecn);			%#
%#########################################
pt_o=.5*(pt_s+pt_r); %I image point Io
pt_iz=.5*(pt_sz+pt_rz);
vec_z=pt_sz-pt_rz;
if(1)
num = dot(pt_iz-pt_o,vec_z);
den = dot(vec_z,vecn);
k   = num /den ;
pt_g(1) = pt_o(1) + k * vecn(1) ;
pt_g(2) = pt_o(2) - k * vecn(2) ;
pt_g(3) = pt_o(3) - k * vecn(3) ;
pt_g
end
if(0)
h0=norm(pt_s-pt_r)/2;
gamma=acos(dot(vecn,vecs));
%shift along the normal
dng=tan(gamma)^2*h0;
pt_g=pt_o+dng*vecn;
end




[sr_x,sr_y,sr_z]=define_line(pt_s,pt_r);

%----   the rays and the interface
dots = linspace(-500,500,nbpts) ;
%--- source ray
line_source_x    = pt_sz(1) + vecs(1) * dots ;
line_source_y    = pt_sz(2) + vecs(2) * dots ;
line_source_z    = pt_sz(3) + vecs(3) * dots ;
%--- receiver ray
line_receiver_x  = pt_rz(1) + vecr(1) * dots ;
line_receiver_y  = pt_rz(2) + vecr(2) * dots ;
line_receiver_z  = pt_rz(3) + vecr(3) * dots ;
%--- normal ray
line_normal_x    = pt_o(1) + vecn(1) * dots ;
line_normal_y    = pt_o(2) + vecn(2) * dots ;
line_normal_z    = pt_o(3) + vecn(3) * dots ;

if(0)
clf
hold on
%<<<<==== PLOTTING EVERYTHING ====>>>>
axis equal
grid
% POINT OF VIEW - CAMERA POSITION
campos([-3251.6 ,  -3137.8 ,   4840.5])

% GIVEN GEOMETRY OF THE PROBLEM	||||||||||||||||||||||||%
%source and receiver end point S and R and image point	%
plot3(pt_s(1), pt_s(2), pt_s(3), 'r*') %So
plot3(pt_r(1), pt_r(2), pt_r(3), 'bv') %Ro
plot3(pt_sz(1),pt_sz(2),pt_sz(3),'ro') %So
plot3(pt_rz(1),pt_rz(2),pt_rz(3),'bo') %Ro
plot3(pt_o(1), pt_o(2), pt_o(3), 'ko') %Io
%source receiver rays 
plot3(line_source_x  ,line_source_y  ,line_source_z  ,'r')
plot3(line_receiver_x,line_receiver_y,line_receiver_z,'b')
%normal ray
plot3(line_normal_x  ,line_normal_y  ,line_normal_z  ,'k-.')
%interface
plot3(sr_x,sr_y,sr_z,'k-.')

plot3(pt_g(1),pt_g(2),pt_g(3),'pk')			%

[vec_ngx,vec_ngy,vec_ngz]=geological_dip_basis(vecn,pt_o,1)


pause
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%===========================
%^^^^^^^^^^^^^^^^^^^^^^^^^^^

%=========================================================
function [line_x,line_y,line_z]=define_line(pt_beg,pt_end)
%--- defines a segment between two given points
nbpts=10;
line_x = linspace(pt_beg(1),pt_end(1),nbpts) ;
line_y = linspace(pt_beg(2),pt_end(2),nbpts) ;
line_z = linspace(pt_beg(3),pt_end(3),nbpts) ;

%=========================================================
function [theta,azim]=vec2angle(vec)
%given the coordinates of a vector, the function
%returns the azimuth and dip of the vector in degrees
%WARNING: doesnt work if input vector = [0 0 1]
vec = vec / norm(vec);
d = sqrt( vec(1)^2 + vec(2)^2 );
if(d==0)
warning off MATLAB:divideByZero
end
theta = atan( vec(3) / d ) * 180 / pi ;
azim  = acos( vec(1) / d ) * 180 / pi ;
if(d==0)
disp('#### azimuth not defined for such vector ####')
end

%=========================================================
function [vec]=angle2vec(theta,azim)
%given the 2 angles of a vector, the function
%returns the coefficients of the vector
vec(1)=-cos(theta)*cos(azim);
vec(2)=cos(theta)*sin(azim);
vec(3)=sin(theta);

%=========================================================
function [vec_ngx,vec_ngy,vec_ngz]=geological_dip_basis(vec_n,pt_center,basis)
%defines the geological dip basis of the plane defined by
%its normal (cf Shan's proposal 2003)
[theta_n,phi_n]=vec2angle(vec_n);
phi_n   = phi_n   / 180 * pi ;
theta_n = theta_n / 180 * pi ;
vec_ngx = [-sin(  phi_n) cos(phi_n) 0   ];
vec_ngy = [ sin(theta_n)*cos(phi_n) ...
            sin(theta_n)*sin(phi_n) ...
	   -cos(theta_n)		];
vec_ngz = [ cos(theta_n)*cos(phi_n) ...
            cos(theta_n)*sin(phi_n) ...
	    sin(theta_n)		];
basis
if(basis==1)
%--- draw basis
radius=300;
pt_ngx = pt_center + radius * vec_ngx ;
pt_ngy = pt_center + radius * vec_ngy ;
pt_ngz = pt_center + radius * vec_ngz ;
[line_ngx_x,line_ngx_y,line_ngx_z]=...
define_line(pt_center,pt_ngx);
plot3(line_ngx_x,line_ngx_y,line_ngx_z,'m')
[line_ngy_x,line_ngy_y,line_ngy_z]=...
define_line(pt_center,pt_ngy);
plot3(line_ngy_x,line_ngy_y,line_ngy_z,'m')
[line_ngz_x,line_ngz_y,line_ngz_z]=...
define_line(pt_center,pt_ngz);
plot3(line_ngz_x,line_ngz_y,line_ngz_z,'m')
end

%vvvvvvvvvvvvvvvvvvvvvvvvvvv
%===========================
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

