function plane_wave_approach(xg,yg,zg,thetas,azims,thetar,azimr)

% This is a model image approach rather than a data approach.
% We start from the image point in angle domain Ig and two plane waves.
% The two plane waves define a line on which is the image point.
% Orthogonal to this plane is the coplanar plane.
		     
clf
%examples of values plane_wave_approach(500,500,500,25,20,55,70)

pt_g=[xg yg zg];

nbpts = 20 ;    %number of points to draw lines

%--- Window
xmin=pt_g(1)-100;
xmax=pt_g(1)+100;
ymin=pt_g(2)-100;
ymax=pt_g(2)+100;
zmin=pt_g(3)-100;
zmax=pt_g(3)+100;

%conversion in radian of alpha and gamma angles
thetas = thetas * pi / 180 ;
thetar = thetar * pi / 180 ;
azimr  = azimr  * pi / 180 ;
azims  = azims  * pi / 180 ;

%|||||||||||||||||||||||||||||||||||||
%|||||||||||||||||||||||||||||||||||||
%<<<<===== ORIGINAL GEOMETRY =====>>>>
%|||||||||||||||||||||||||||||||||||||
%|||||||||||||||||||||||||||||||||||||

%<<<<==== SOURCE AND RECEIVER PLANE WAVES ====>>>>

%---- ray vectors for source, receiver, normal
vecs=angle2vec(thetas,azims);
vecr=angle2vec(thetar,azimr);

%--- S plane wave: orthogonal to S ray and containing Ig(xg,yg,zg)
[plane_sx,plane_sy,plane_sz]=...
define_plane(vecs,pt_g,xmin,pt_g(1),ymin,ymax);
%--- R plane wave: orthogonal to R ray and containing Ig(xg,yg,zg)
[plane_rx,plane_ry,plane_rz]=...
define_plane(vecr,pt_g,pt_g(1),xmax,ymin,ymax);

%--- Intersection of the two plane waves
vec_int = cross(vecs,vecr);
[line_intx,line_inty,line_intz]=define_line_1pt(...
vec_int,pt_g,xmin,xmax,ymin,ymax,zmin,zmax);

%<<<<==== NORMAL ====>>>>
vecn = vecs + vecr ; %normal ray direction
vecn = vecn / norm(vecn) ; %normalization

[line_nx,line_ny,line_nz]=define_line_1pt(...
vecn,pt_g,xmin,xmax,ymin,ymax,zmin,zmax);

%<<<<==== HORIZONTAL PLANE CONTAINING Ig  ====>>>>
[plane_hx,plane_hy,plane_hz]=...
define_plane([0 0 1],pt_g,xmin,xmax,ymin,ymax);

%<<<<==== COPLANAR PLANE ====>>>>
vec_n=cross(vecn,vec_int);
[plane_copx,plane_copy,plane_copz]=...
define_plane(vec_int,pt_g,xmin,pt_g(1),ymin,ymax);

%<<<<==== INTERFACE ====>>>>
[plane_intx,plane_inty,plane_intz]=...
define_plane(vecn,pt_g,xmin,xmax(1),ymin,ymax);
%--- geological-dip basis
[vec_ngx,vec_ngy,vec_ngz]=geological_dip_basis(vecn,pt_g,1);
%--- geological-dip azimuth of coplanar plane
geol_azim_ref = acos(dot(vec_ngx,vec_int))*180/pi ;

		 
%|||||||||||||||||||||||||||||||||||||
%|||||||||||||||||||||||||||||||||||||
%<<<<==== NUMERICAL VALUES    ====>>>>
%|||||||||||||||||||||||||||||||||||||
%|||||||||||||||||||||||||||||||||||||
%---Angles of normal
disp('### Angles of the normal (or of the interface):')
[theta_n,azim_n]=vec2angle(vecn)
%---Aperture angle
disp('### Aperture angle')
gamma = dot(vecr,vecs)*180/pi/2
%---Geological-dip azimuth of the coplanar plane
disp('### Geological-dip azimuth of the coplanar plane:')
geol_azim_ref
%---Angles of coplanar plane
disp('### Angles of the plane waves common line (or of the coplanar plane):')
[theta_cop_n,azim_cop_n]=vec2angle(vec_n)

%|||||||||||||||||||||||||||||||||||||
%<<<<==== PLOTTING EVERYTHING ====>>>>
%|||||||||||||||||||||||||||||||||||||
%|||||||||||||||||||||||||||||||||||||

hold on
colormap(hsv)
shading interp
%axis vis3d off
axis([xmin xmax ymin ymax zmin zmax])
axis equal
grid
xlabel('x')
ylabel('y')
zlabel('z')
% POINT OF VIEW - CAMERA POSITION
campos([-3251.6 ,  -3137.8 ,   4840.5])

%%%%%%==== POINTS ===%%%%%%
% Ig image point in angle domain
plot3(pt_g(1),pt_g(2),pt_g(3),'ko')
%%%%%%==== LINES ===%%%%%%
% Normal
plot3(line_nx,line_ny,line_nz,'k-.')
% intersection of S and R planes
plot3(line_intx,line_inty,line_intz,'k')
%%%%%%==== PLANES ===%%%%%%
% S and R plane wave
surf(plane_sx,plane_sy,plane_sz)
surf(plane_rx,plane_ry,plane_rz)
% coplanar plane 
surf(plane_copx,plane_copy,plane_copz)
% horizontal plane 
%surf(plane_hx,plane_hy,plane_hz)
% interface
%surf(plane_intx,plane_inty,plane_intz)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ANIMATION WITH ORIGINAL AND COPLANAR RAYS ||||||||||||%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(0)							%
axis vis3d off						%
%for x = 5000:-100:-3251.6				%
%  campos([x ,  -3137.8 ,   4840.5])			%
%  drawnow						%
%end							%
for x = 3*pi/2:.01:6*pi/2				%
  campos([3000*cos(x) ,  3000*sin(x) ,  5000 ])		%
  drawnow						%
end							%
end							%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

shading interp

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%===========================
%^^^^^^^^^^^^^^^^^^^^^^^^^^^

%=========================================================
function [plane_x,plane_y,plane_z]=define_plane(...
vecn,pt,xmin,xmax,ymin,ymax)
%a plane is defined by a normal (vecn) and a point (pt)
x=linspace(xmin,xmax,10);
y=linspace(ymin,ymax,10);
[plane_x,plane_y]=meshgrid(x,y);
 plane_z = pt(3) - (vecn(1)*(plane_x-pt(1))+...
                    vecn(2)*(plane_y-pt(2)) )/vecn(3);
%=========================================================
function [line_x,line_y,line_z]=define_line_1pt(...
vec,pt,xmin,xmax,ymin,ymax,zmin,zmax)
%--- defines a line with one vector and one point
nbpts=100;
line_x =  pt(1) + vec(1) * linspace(xmin-pt(1),xmax-pt(1),nbpts) ;
line_y =  pt(2) + vec(2) * linspace(ymin-pt(2),ymax-pt(2),nbpts) ;
line_z =  pt(3) + vec(3) * linspace(zmin-pt(3),zmax-pt(3),nbpts) ;
%=========================================================
function [line_x,line_y,line_z]=define_line_2pt(pt_beg,pt_end)
%--- defines a segment between two given points
nbpts=10;
line_x = linspace(pt_beg(1),pt_end(1),nbpts) ;
line_y = linspace(pt_beg(2),pt_end(2),nbpts) ;
line_z = linspace(pt_beg(3),pt_end(3),nbpts) ;
%=========================================================
function [theta,azim]=vec2angle(vec)
%given the coordinates of a vector, the function
%returns the azimuth and dip of the vector in degrees
%WARNING: doesnt work if input vector = [0 0 1]
vec = vec / norm(vec);
d = sqrt( vec(1)^2 + vec(2)^2 );
if(d==0)
warning off MATLAB:divideByZero
end
theta = atan( vec(3) / d ) * 180 / pi ;
azim  = acos( vec(1) / d ) * 180 / pi ;
if(d==0)
disp('#### azimuth not defined for such vector ####')
end
%=========================================================
function [vec]=angle2vec(theta,azim)
%given the 2 angles of a vector, the function
%returns the coefficients of the vector
vec(1)=-cos(theta)*cos(azim);
vec(2)=cos(theta)*sin(azim);
vec(3)=sin(theta);
%=========================================================
function [vec_ngx,vec_ngy,vec_ngz]=geological_dip_basis(vec_n,pt,basis)
%defines the geological dip basis of the plane defined by
%its normal (Cf Shan's proposal 2003)
[theta_n,phi_n]=vec2angle(vec_n);
phi_n   = phi_n   / 180 * pi ;
theta_n = theta_n / 180 * pi ;
vec_ngx = [-sin(  phi_n) cos(phi_n) 0   ];
vec_ngy = [ sin(theta_n)*cos(phi_n) ...
            sin(theta_n)*sin(phi_n) ...
	   -cos(theta_n)		];
vec_ngz = [ cos(theta_n)*cos(phi_n) ...
            cos(theta_n)*sin(phi_n) ...
	    sin(theta_n)		];

if(basis)
radius=100;
pt_ngx = pt + radius * vec_ngx ;
pt_ngy = pt + radius * vec_ngy ;
pt_ngz = pt + radius * vec_ngz ;
[line_ngx_x,line_ngx_y,line_ngx_z]=...
define_line_2pt(pt,pt_ngx);
[line_ngy_x,line_ngy_y,line_ngy_z]=...
define_line_2pt(pt,pt_ngy);
[line_ngz_x,line_ngz_y,line_ngz_z]=...
define_line_2pt(pt,pt_ngz);
hold on
plot3(line_ngx_x,line_ngx_y,line_ngx_z,'mo')
plot3(line_ngy_x,line_ngy_y,line_ngy_z,'mo')
plot3(line_ngz_x,line_ngz_y,line_ngz_z,'mo')
end

%vvvvvvvvvvvvvvvvvvvvvvvvvvv
%===========================
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
